@@ -1,29 +1,30 @@
-/*-*-C-*-
- * Copyright 2006  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2006--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CU_VAREF_H
-#define CU_VAREF_H
+#ifndef CU_VA_REF_H
+#define CU_VA_REF_H
 
 #include <cu/fwd.h>
 #include <cu/conf.h>
 #include <stdarg.h>
 
 CU_BEGIN_DECLARATIONS
+/** \defgroup cu_va_ref_h cu/va_ref.h: Passing \c va_list as a Reference
+ ** @{ \ingroup cu_util_mod */
 
 #ifdef CUCONF_VA_LIST_IS_REF
 
@@ -41,6 +42,7 @@ typedef va_list *cu_va_ref_t;
 
 #endif
 
+/** @} */
 CU_END_DECLARATIONS
 
 #endif
