@@ -1,3 +1,20 @@
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2007  Petter Urkedal <urkedal@nbi.dk>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include <cucon/pmap.h>
 #include <cu/memory.h>
 #include <time.h>
@@ -46,7 +63,7 @@ bench_em(int N)
 #endif
 	t1 += clock();
 	if (j == 0)
-	    cucon_pmap_show_stats(pmap);
+	    cucon_pmap_dump_stats(pmap, stdout);
     }
     printf("%8d%16lg%16lg\n", N,
 	   t0/((double)CLOCKS_PER_SEC * N * J),
