@@ -1,24 +1,24 @@
-/*-*-C-*-
- * Copyright 2003--2004  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2003--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <cucon/rbtree.h>
 #include <cu/memory.h>
 #include <cu/int.h>
+#include <cu/ptr.h>
 #include <assert.h>
 #include <string.h>
 
@@ -49,7 +49,7 @@
 #define IS_RED(node) !IS_BLACK(node)
 
 void
-cucon_rbtree_cct(cucon_rbtree_t tree)
+cucon_rbtree_init(cucon_rbtree_t tree)
 {
     tree->root = NULL;
 }
@@ -57,7 +57,7 @@ cucon_rbtree_cct(cucon_rbtree_t tree)
 cucon_rbtree_t
 cucon_rbtree_new(void)
 {
-    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree_s);
+    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree);
     tree->root = NULL;
     return tree;
 }
@@ -71,18 +71,18 @@ cucon_rbtree_cct_copy_ctive(cucon_rbtree_t tree, cucon_rbtree_t src)
 cucon_rbtree_t
 cucon_rbtree_new_copy_ctive(cucon_rbtree_t src)
 {
-    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree_s);
+    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree);
     tree->root = src->root;
     return tree;
 }
 
 CU_SINLINE cucon_rbnode_t
-copy_ptrnode(cucon_rbnode_t src)
+_copy_ptrnode(cucon_rbnode_t src)
 {
-    cucon_rbnode_t dst = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode_s)
+    cucon_rbnode_t dst = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode)
 				   + sizeof(void *));
     dst->is_black = src->is_black;
-    *(void **)cucon_rbnode_get_mem(dst) = *(void **)cucon_rbnode_get_mem(src);
+    *(void **)cucon_rbnode_mem(dst) = *(void **)cucon_rbnode_mem(src);
     return dst;
 }
 
@@ -90,7 +90,7 @@ cucon_rbnode_t
 cucon_rbnode_new_copy(cucon_rbnode_t src, size_t slot_size)
 {
     cucon_rbnode_t dst
-	= cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode_s) + slot_size);
+	= cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode) + slot_size);
     dst->is_black = src->is_black;
 #ifndef NDEBUG
     dst->left = (void *)-1;
@@ -120,7 +120,7 @@ cucon_rbtree_cct_copy_node(cucon_rbtree_t dst, cucon_rbtree_t src,
 cucon_rbtree_t
 cucon_rbtree_new_copy_node(cucon_rbtree_t src, cucon_rbnode_copier_t node_new_copy)
 {
-    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree_s);
+    cucon_rbtree_t tree = cu_gnew(struct cucon_rbtree);
     cucon_rbtree_cct_copy_node(tree, src, node_new_copy);
     return tree;
 }
@@ -129,7 +129,7 @@ static void
 cuconP_rbnode_copy_rec_ptr(cucon_rbnode_t src, cucon_rbnode_t *dst)
 {
     while (src) {
-	*dst = copy_ptrnode(src);
+	*dst = _copy_ptrnode(src);
 	cuconP_rbnode_copy_rec_ptr(LEFT(src), &LEFT(*dst));
 	src = RIGHT(src);
 	dst = &RIGHT(*dst);
@@ -145,43 +145,41 @@ cucon_rbtree_cct_copy_ptr(cucon_rbtree_t dst, cucon_rbtree_t src)
 cucon_rbtree_t
 cucon_rbtree_new_copy_ptr(cucon_rbtree_t src)
 {
-    cucon_rbtree_t dst = cu_gnew(struct cucon_rbtree_s);
+    cucon_rbtree_t dst = cu_gnew(struct cucon_rbtree);
     cuconP_rbnode_copy_rec_ptr(src->root, &dst->root);
     return dst;
 }
 
-cu_clos_def(merge_ptr_cb, void cu_proto(void *value),
+cu_clos_def(_merge_ptr_cb, cu_prot(void, void *value),
 	    (cucon_rbtree_t dst; cu_clop(cmp, int, void *, void *);))
 {
-    cu_clos_self(merge_ptr_cb);
-    cucon_rbtree_insert_ptr(self->dst, self->cmp, value);
+    cu_clos_self(_merge_ptr_cb);
+    cucon_rbtree_insert2p_ptr(self->dst, self->cmp, value);
 }
 void
-cucon_rbtree_merge_ptr(cucon_rbtree_t dst, cucon_rbtree_t src,
+cucon_rbtree_merge2p(cucon_rbtree_t dst, cucon_rbtree_t src,
 		       cu_clop(cmp, int, void *, void *))
 {
-    merge_ptr_cb_t cb;
-    cu_clos_cct(&cb, merge_ptr_cb);
+    _merge_ptr_cb_t cb;
     cb.dst = dst;
     cb.cmp = cmp;
-    cucon_rbtree_for_ptr(src, cu_clos_ref(&cb));
+    cucon_rbtree_iter_ptr(src, _merge_ptr_cb_prep(&cb));
 }
 
-cu_clos_def(merge_ptr_ctive_cb, void cu_proto(void *value),
+cu_clos_def(_merge_ptr_ctive_cb, cu_prot(void, void *value),
 	    (cucon_rbtree_t dst; cu_clop(cmp, int, void *, void *);))
 {
-    cu_clos_self(merge_ptr_ctive_cb);
-    cucon_rbtree_insert_ptr_ctive(self->dst, self->cmp, value);
+    cu_clos_self(_merge_ptr_ctive_cb);
+    cucon_rbtree_insert2p_ptr(self->dst, self->cmp, value);
 }
 void
-cucon_rbtree_merge_ptr_ctive(cucon_rbtree_t dst, cucon_rbtree_t src,
+cucon_rbtree_cmerge2p(cucon_rbtree_t dst, cucon_rbtree_t src,
 			     cu_clop(cmp, int, void *, void *))
 {
-    merge_ptr_ctive_cb_t cb;
-    cu_clos_cct(&cb, merge_ptr_ctive_cb);
+    _merge_ptr_ctive_cb_t cb;
     cb.dst = dst;
     cb.cmp = cmp;
-    cucon_rbtree_for_ptr(src, cu_clos_ref(&cb));
+    cucon_rbtree_iter_ptr(src, _merge_ptr_ctive_cb_prep(&cb));
 }
 
 #ifdef CHECK_TREE
@@ -367,8 +365,8 @@ cuconP_rbtree_rebalance_dtive(cucon_rbnode_t *sp)
 }
 
 cu_bool_t
-cucon_rbtree_insert_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *),
-			size_t slot_size, cu_ptr_ptr_t slot)
+cucon_rbtree_insert1m_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *),
+			  size_t slot_size, cu_ptr_ptr_t slot)
 {
     size_t tree_size = cucon_rbtree_size(tree);
     size_t stack_size = 2*cu_ulong_floor_log2(tree_size + 1) + 3;
@@ -386,18 +384,18 @@ cucon_rbtree_insert_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *),
 	assert(IS_BLACK(x));
     while (x != NULL) {
 	*--sp = x;
-	cmp = cu_call(compare, cucon_rbnode_get_mem(x));
+	cmp = cu_call(compare, cucon_rbnode_mem(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
 	    x = x->right;
 	else {
-	    *(void **)slot = cucon_rbnode_get_mem(x);
+	    *(void **)slot = cucon_rbnode_mem(x);
 	    assert(tree->root->is_black == tree_size);
 	    return cu_false;
 	}
     }
-    *--sp = x = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode_s)
+    *--sp = x = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode)
 			   + slot_size);
     if (sp[1]) {
 	if (cmp < 0)
@@ -407,7 +405,7 @@ cucon_rbtree_insert_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *),
     }
     else
 	tree->root = x;
-    *(void **)slot = cucon_rbnode_get_mem(x);
+    *(void **)slot = cucon_rbnode_mem(x);
     cuconP_rbtree_rebalance_dtive(sp);
     tree->root = *sp_root;
     tree->root->is_black = ++tree_size;
@@ -419,7 +417,7 @@ cucon_rbtree_insert_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *),
 }
 
 cu_bool_t
-cucon_rbtree_insert_by_ptr(cucon_rbtree_t tree,
+cucon_rbtree_insert2p_node(cucon_rbtree_t tree,
 			   cu_clop(compare, int, void *, void *),
 			   void *key,
 			   size_t node_size, size_t node_offset,
@@ -441,7 +439,7 @@ cucon_rbtree_insert_by_ptr(cucon_rbtree_t tree,
 	assert(IS_BLACK(x));
     while (x != NULL) {
 	*--sp = x;
-	cmp = cu_call(compare, key, *(void **)cucon_rbnode_get_mem(x));
+	cmp = cu_call(compare, key, *(void **)cucon_rbnode_mem(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
@@ -452,7 +450,7 @@ cucon_rbtree_insert_by_ptr(cucon_rbtree_t tree,
 	    return cu_false;
 	}
     }
-    x = cu_galloc(node_size) + node_offset;
+    x = cu_ptr_add(cu_galloc(node_size), node_offset);
     *--sp = x;
     if (sp[1]) {
 	if (cmp < 0)
@@ -462,7 +460,7 @@ cucon_rbtree_insert_by_ptr(cucon_rbtree_t tree,
     }
     else
 	tree->root = x;
-    *(void **)cucon_rbnode_get_mem(x) = key;
+    *(void **)cucon_rbnode_mem(x) = key;
     *node_out = x;
     cuconP_rbtree_rebalance_dtive(sp);
     tree->root = *sp_root;
@@ -475,16 +473,16 @@ cucon_rbtree_insert_by_ptr(cucon_rbtree_t tree,
 }
 
 cu_bool_t
-cucon_rbtree_insert_ptr(cucon_rbtree_t tree,
-			cu_clop(compare, int, void *, void *),
-			cu_ptr_ptr_t ptr_io)
+cucon_rbtree_insert2p_ptr(cucon_rbtree_t tree,
+			  cu_clop(compare, int, void *, void *),
+			  cu_ptr_ptr_t ptr_io)
 {
     cucon_rbnode_t node;
-    if (cucon_rbtree_insert_by_ptr(
+    if (cucon_rbtree_insert2p_node(
 		tree, compare, *(void **)ptr_io,
-		sizeof(struct cucon_rbnode_s) + sizeof(void *), 0,
+		sizeof(struct cucon_rbnode) + sizeof(void *), 0,
 		&node)) {
-	*(void **)ptr_io = cucon_rbnode_get_ptr(node);
+	*(void **)ptr_io = cucon_rbnode_ptr(node);
 	return cu_true;
     }
     else
@@ -492,11 +490,11 @@ cucon_rbtree_insert_ptr(cucon_rbtree_t tree,
 }
 
 cu_bool_t
-cucon_rbtree_insert_mem_ctive(cucon_rbtree_t tree,
-			      cu_clop(slot_compare, int, void *),
-			      cucon_rbnode_copier_t node_new_copy,
-			      size_t slot_size,
-			      cu_ptr_ptr_t slot)
+cucon_rbtree_cinsert1m_mem(cucon_rbtree_t tree,
+			   cu_clop(slot_compare, int, void *),
+			   cucon_rbnode_copier_t node_new_copy,
+			   size_t slot_size,
+			   cu_ptr_ptr_t slot)
 {
     size_t tree_size = cucon_rbtree_size(tree);
     size_t stack_size = 2*cu_ulong_floor_log2(tree_size + 1) + 3;
@@ -511,29 +509,29 @@ cucon_rbtree_insert_mem_ctive(cucon_rbtree_t tree,
     /* Insert */
     x = tree->root;
     if (!x) {
-	x = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode_s) + slot_size);
+	x = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode) + slot_size);
 	SET_LEFT(x, NULL);
 	SET_RIGHT(x, NULL);
 	x->is_black = 1;
-	*(void **)slot = cucon_rbnode_get_mem(x);
+	*(void **)slot = cucon_rbnode_mem(x);
 	tree->root = x;
 	return cu_true;
     }
     assert(IS_BLACK(x));
     while (x != NULL) {
 	*--sp = x;
-	cmp = cu_call(slot_compare, cucon_rbnode_get_mem(x));
+	cmp = cu_call(slot_compare, cucon_rbnode_mem(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
 	    x = x->right;
 	else {
-	    *(void **)slot = cucon_rbnode_get_mem(x);
+	    *(void **)slot = cucon_rbnode_mem(x);
 	    assert(tree->root->is_black == tree_size);
 	    return cu_false;
 	}
     }
-    *--sp = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode_s) + slot_size);
+    *--sp = cu_galloc(CU_ALIGNED_SIZEOF(struct cucon_rbnode) + slot_size);
     sp_leaf = sp;
     x1 = cu_call(node_new_copy, sp_root[0]);
     for (sp = sp_root - 1; sp != sp_leaf; --sp) {
@@ -560,7 +558,7 @@ cucon_rbtree_insert_mem_ctive(cucon_rbtree_t tree,
 	SET_LEFT(x1, LEFT(sp[1]));
     }
     sp[1] = x1;
-    *(void **)slot = cucon_rbnode_get_mem(x);
+    *(void **)slot = cucon_rbnode_mem(x);
 
 #ifdef CHECK_TREE
     tree->root = sp_root[0];
@@ -626,29 +624,28 @@ cucon_rbtree_insert_mem_ctive(cucon_rbtree_t tree,
 cu_clop_def(ptrnode_new_copy, cucon_rbnode_t, cucon_rbnode_t src)
 {
     cucon_rbnode_t node = cucon_rbnode_new_copy(src, sizeof(void *));
-    *(void **)cucon_rbnode_get_mem(node) = *(void **)cucon_rbnode_get_mem(src);
+    *(void **)cucon_rbnode_mem(node) = *(void **)cucon_rbnode_mem(src);
     return node;
 }
 
-cu_clos_def(ptrslot_compare,
-	    int cu_proto(void *slot),
+cu_clos_def(_ptrslot_compare,
+	    cu_prot(int, void *slot),
 	    (cu_clop(ptr_compare, int, void *, void *); void *ptr;))
 {
-    cu_clos_self(ptrslot_compare);
+    cu_clos_self(_ptrslot_compare);
     return cu_call(self->ptr_compare, self->ptr, *(void **)slot);
 }
 cu_bool_t
-cucon_rbtree_insert_ptr_ctive(cucon_rbtree_t tree,
-			      cu_clop(ptr_compare, int, void *, void *),
-			      cu_ptr_ptr_t ptr_io)
+cucon_rbtree_cinsert2p_ptr(cucon_rbtree_t tree,
+			   cu_clop(ptr_compare, int, void *, void *),
+			   cu_ptr_ptr_t ptr_io)
 {
     void **slot;
-    ptrslot_compare_t cmp;
-    cu_clos_cct(&cmp, ptrslot_compare);
+    _ptrslot_compare_t cmp;
     cmp.ptr_compare = ptr_compare;
     cmp.ptr = *(void **)ptr_io;
-    if (cucon_rbtree_insert_mem_ctive(
-	    tree, cu_clos_ref(&cmp), ptrnode_new_copy,
+    if (cucon_rbtree_cinsert1m_mem(
+	    tree, _ptrslot_compare_prep(&cmp), ptrnode_new_copy,
 	    sizeof(void *), &slot)) {
 	*slot = *(void **)ptr_io;
 	return cu_true;
@@ -659,21 +656,21 @@ cucon_rbtree_insert_ptr_ctive(cucon_rbtree_t tree,
     }
 }
 
-cu_clos_def(imem_slot_cmp,
-	    int cu_proto(void *slot),
+cu_clos_def(_imem_slot_cmp,
+	    cu_prot(int, void *slot),
     ( size_t key_size;
       void *key_data; ))
 {
-    cu_clos_self(imem_slot_cmp);
+    cu_clos_self(_imem_slot_cmp);
     return memcmp(self->key_data, slot, self->key_size);
 }
-cu_clos_def(imem_node_copy,
-	    cucon_rbnode_t cu_proto(cucon_rbnode_t node),
+cu_clos_def(_imem_node_copy,
+	    cu_prot(cucon_rbnode_t, cucon_rbnode_t node),
     ( size_t slot_size; ))
 {
-    cu_clos_self(imem_node_copy);
+    cu_clos_self(_imem_node_copy);
     cucon_rbnode_t new_node = cucon_rbnode_new_copy(node, self->slot_size);
-    memcpy(cucon_rbnode_get_mem(new_node), cucon_rbnode_get_mem(node),
+    memcpy(cucon_rbnode_mem(new_node), cucon_rbnode_mem(node),
 	   self->slot_size);
     return new_node;
 }
@@ -683,17 +680,16 @@ cucon_rbtree_insert_imem_ctive(cucon_rbtree_t tree,
 			       size_t key_size, void *key,
 			       size_t slot_size, cu_ptr_ptr_t slot_o)
 {
-    imem_slot_cmp_t cmp;
-    imem_node_copy_t cpy;
+    _imem_slot_cmp_t cmp;
+    _imem_node_copy_t cpy;
     void *new_slot;
-    cu_clos_cct(&cmp, imem_slot_cmp);
     cmp.key_size = key_size;
     cmp.key_data = key;
-    cu_clos_cct(&cpy, imem_node_copy);
     cpy.slot_size = slot_size;
-    if (cucon_rbtree_insert_mem_ctive(tree,
-				      cu_clos_ref(&cmp), cu_clos_ref(&cpy),
-				      slot_size, &new_slot)) {
+    if (cucon_rbtree_cinsert1m_mem(tree,
+				   _imem_slot_cmp_prep(&cmp),
+				   _imem_node_copy_prep(&cpy),
+				   slot_size, &new_slot)) {
 	memcpy(new_slot, key, key_size);
 	*(void **)slot_o = new_slot;
 	return cu_true;
@@ -710,8 +706,8 @@ cucon_rbtree_insert_imem_ctive(cucon_rbtree_t tree,
  * ----- */
 
 static void
-rbtree_erase(cucon_rbtree_t tree,
-	     cucon_rbnode_t *sp_root, cucon_rbnode_t *sp_z, cucon_rbnode_t z)
+_rbtree_erase(cucon_rbtree_t tree,
+	      cucon_rbnode_t *sp_root, cucon_rbnode_t *sp_z, cucon_rbnode_t z)
 {
     cucon_rbnode_t *sp;
     cucon_rbnode_t x, y;
@@ -890,21 +886,21 @@ skip_move:
     }									\
     if (z == NULL)							\
 	return NULL;							\
-    rbtree_erase(tree, sp_root, sp_z, z);
+    _rbtree_erase(tree, sp_root, sp_z, z);
 
 cucon_rbnode_t
-cucon_rbtree_erase_by_ptr(cucon_rbtree_t tree,
-			  cu_clop(compare, int, void *, void *), void *key)
+cucon_rbtree_erase2p(cucon_rbtree_t tree,
+		     cu_clop(compare, int, void *, void *), void *key)
 {
-    RBTREE_ERASE(tree, z, cu_call(compare, key, cucon_rbnode_get_ptr(z)));
+    RBTREE_ERASE(tree, z, cu_call(compare, key, cucon_rbnode_ptr(z)));
     return z;
 }
 
 void *
-cucon_rbtree_erase(cucon_rbtree_t tree, cu_clop(compare, int, void *))
+cucon_rbtree_erase1m(cucon_rbtree_t tree, cu_clop(compare, int, void *))
 {
-    RBTREE_ERASE(tree, z, cu_call(compare, cucon_rbnode_get_mem(z)));
-    return cucon_rbnode_get_mem(z);
+    RBTREE_ERASE(tree, z, cu_call(compare, cucon_rbnode_mem(z)));
+    return cucon_rbnode_mem(z);
 }
 
 
@@ -913,12 +909,12 @@ cucon_rbtree_erase(cucon_rbtree_t tree, cu_clop(compare, int, void *))
  * ---- */
 
 cucon_rbnode_t
-cucon_rbtree_find_by_ptr(cucon_rbtree_t tree,
+cucon_rbtree_find2p_node(cucon_rbtree_t tree,
 			 cu_clop(compare, int, void *, void *), void *key)
 {
     cucon_rbnode_t x = tree->root;
     while (x != NULL) {
-	int cmp = cu_call(compare, key, cucon_rbnode_get_ptr(x));
+	int cmp = cu_call(compare, key, cucon_rbnode_ptr(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
@@ -930,39 +926,78 @@ cucon_rbtree_find_by_ptr(cucon_rbtree_t tree,
 }
 
 void *
-cucon_rbtree_find_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *))
+cucon_rbtree_find1m_mem(cucon_rbtree_t tree, cu_clop(compare, int, void *))
 {
     cucon_rbnode_t x = tree->root;
     while (x != NULL) {
-	int cmp = cu_call(compare, cucon_rbnode_get_mem(x));
+	int cmp = cu_call(compare, cucon_rbnode_mem(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
 	    x = x->right;
 	else
-	    return cucon_rbnode_get_mem(x);
+	    return cucon_rbnode_mem(x);
     }
     return NULL;
 }
 
 void *
-cucon_rbtree_find_ptr(cucon_rbtree_t tree,
-		      cu_clop(compare, int, void *, void *),
-		      void *key)
+cucon_rbtree_find2p_ptr(cucon_rbtree_t tree,
+			cu_clop(compare, int, void *, void *),
+			void *key)
 {
     cucon_rbnode_t x = tree->root;
     while (x != NULL) {
-	int cmp = cu_call(compare, key, cucon_rbnode_get_ptr(x));
+	int cmp = cu_call(compare, key, cucon_rbnode_ptr(x));
 	if (cmp < 0)
 	    x = x->left;
 	else if (cmp > 0)
 	    x = x->right;
 	else
-	    return cucon_rbnode_get_ptr(x);
+	    return cucon_rbnode_ptr(x);
     }
     return NULL;
 }
 
+void
+cucon_rbtree_nearest2p(cucon_rbtree_t tree,
+		       cu_clop(cmp2, int, void *, void *), void *key,
+		       cucon_rbnode_t *below_out,
+		       cucon_rbnode_t *equal_out,
+		       cucon_rbnode_t *above_out)
+{
+    cucon_rbnode_t below = NULL, equal = NULL, above = NULL;
+    cucon_rbnode_t x = tree->root;
+    while (x != NULL) {
+	int cmp = cu_call(cmp2, key, cucon_rbnode_ptr(x));
+	if (cmp < 0) {
+	    above = x;
+	    x = x->left;
+	}
+	else if (cmp > 0) {
+	    below = x;
+	    x = x->right;
+	}
+	else {
+	    equal = x;
+	    x = equal->left;
+	    while (x != NULL) {
+		below = x;
+		x = x->right;
+	    }
+	    x = equal->right;
+	    while (x != NULL) {
+		above = x;
+		x = x->left;
+	    }
+	    break;
+	}
+    }
+    *above_out = above;
+    *equal_out = equal;
+    *below_out = below;
+}
+
 void
 cuconP_rbnode_dump_as_graphviz(cucon_rbnode_t x,
 			       cu_clop(label, void, void *, FILE *),
@@ -972,27 +1007,29 @@ cuconP_rbnode_dump_as_graphviz(cucon_rbnode_t x,
 tail_rec:
     y = x->left;
     if (cu_clop_is_null(label))
-	fprintf(out, "v%p[color=%s];\n", x, IS_RED(x)? "red" : "black");
+	fprintf(out, "v%p[color=%s];\n", (void *)x,
+		IS_RED(x)? "red" : "black");
     else {
-	fprintf(out, "v%p[color=%s,label=\"", x, IS_RED(x)? "red" : "black");
-	cu_call(label, cucon_rbnode_get_mem(x), out);
+	fprintf(out, "v%p[color=%s,label=\"", (void *)x,
+		IS_RED(x)? "red" : "black");
+	cu_call(label, cucon_rbnode_mem(x), out);
 	fprintf(out, "\"];\n");
     }
     if (y) {
-	fprintf(out, "v%p->v%p;\n", x, y);
+	fprintf(out, "v%p->v%p;\n", (void *)x, (void *)y);
 	cuconP_rbnode_dump_as_graphviz(y, label, out);
     }
     else
 	fprintf(out, "v%p->v%p; v%p[label=\"\"];\n",
-		x, (char *)x + 1, (char *)x + 1);
+		(void *)x, (char *)x + 1, (char *)x + 1);
     y = x->right;
     if (y) {
-	fprintf(out, "v%p->v%p;\n", x, y);
+	fprintf(out, "v%p->v%p;\n", (void *)x, (void *)y);
 	x = y;
 	goto tail_rec;
     }
     else
-	fprintf(out, "v%p->v%p; v%p[label=\"\"];\n", x,
+	fprintf(out, "v%p->v%p; v%p[label=\"\"];\n", (void *)x,
 		(char *)x + 2, (char *)x + 2);
 }
 
@@ -1008,35 +1045,67 @@ cucon_rbtree_dump_as_graphviz(cucon_rbtree_t tree,
 }
 
 static void
-cuconP_rbnode_for_mem(cucon_rbnode_t node, cu_clop(cb, void, void *))
+cuconP_rbnode_iter_mem(cucon_rbnode_t node, cu_clop(cb, void, void *))
 {
     while (node) {
-	cuconP_rbnode_for_mem(node->left, cb);
+	cuconP_rbnode_iter_mem(node->left, cb);
 	cu_call(cb, CU_ALIGNED_PTR_END(node));
 	node = node->right;
     }
 }
 
 void
-cucon_rbtree_for_mem(cucon_rbtree_t tree, cu_clop(cb, void, void *))
+cucon_rbtree_iter_mem(cucon_rbtree_t tree, cu_clop(cb, void, void *))
 {
-    cuconP_rbnode_for_mem(tree->root, cb);
+    cuconP_rbnode_iter_mem(tree->root, cb);
 }
 
 static void
-cuconP_rbnode_for_ptr(cucon_rbnode_t node, cu_clop(cb, void, void *))
+cuconP_rbnode_iter_ptr(cucon_rbnode_t node, cu_clop(cb, void, void *))
 {
     while (node) {
-	cuconP_rbnode_for_ptr(node->left, cb);
+	cuconP_rbnode_iter_ptr(node->left, cb);
 	cu_call(cb, *(void**)CU_ALIGNED_PTR_END(node));
 	node = node->right;
     }
 }
 
 void
-cucon_rbtree_for_ptr(cucon_rbtree_t tree, cu_clop(cb, void, void *))
+cucon_rbtree_iter_ptr(cucon_rbtree_t tree, cu_clop(cb, void, void *))
 {
-    cuconP_rbnode_for_ptr(tree->root, cb);
+    cuconP_rbnode_iter_ptr(tree->root, cb);
+}
+
+static void
+cuconP_rbnode_rev_iter_mem(cucon_rbnode_t node, cu_clop(cb, void, void *))
+{
+    while (node) {
+	cuconP_rbnode_rev_iter_mem(node->right, cb);
+	cu_call(cb, CU_ALIGNED_PTR_END(node));
+	node = node->left;
+    }
+}
+
+void
+cucon_rbtree_rev_iter_mem(cucon_rbtree_t tree, cu_clop(cb, void, void *))
+{
+    cuconP_rbnode_rev_iter_mem(tree->root, cb);
+}
+
+static void
+cuconP_rbnode_rev_iter_ptr(cucon_rbnode_t node, cu_clop(cb, void, void *))
+{
+    while (node) {
+	cuconP_rbnode_rev_iter_ptr(node->right, cb);
+	cu_call(cb, *(void**)CU_ALIGNED_PTR_END(node));
+	node = node->left;
+    }
+}
+
+void
+cucon_rbtree_rev_iter_ptr(cucon_rbtree_t tree, cu_clop(cb, void, void *))
+{
+    cuconP_rbnode_rev_iter_ptr(tree->root, cb);
 }
 
 static cu_bool_t
@@ -1074,3 +1143,39 @@ cucon_rbtree_conj_ptr(cucon_rbtree_t tree, cu_clop(cb, cu_bool_t, void *))
 {
     return cuconP_rbnode_conj_ptr(tree->root, cb);
 }
+
+static cu_bool_t
+cuconP_rbnode_rev_conj_mem(cucon_rbnode_t node, cu_clop(cb, cu_bool_t, void *))
+{
+    while (node) {
+	if (!cuconP_rbnode_rev_conj_mem(node->right, cb))
+	    return cu_false;
+	if (!cu_call(cb, CU_ALIGNED_PTR_END(node)))
+	    return cu_false;
+	node = node->left;
+    }
+    return cu_true;
+}
+cu_bool_t
+cucon_rbtree_rev_conj_mem(cucon_rbtree_t tree, cu_clop(cb, cu_bool_t, void *))
+{
+    return cuconP_rbnode_rev_conj_mem(tree->root, cb);
+}
+
+static cu_bool_t
+cuconP_rbnode_rev_conj_ptr(cucon_rbnode_t node, cu_clop(cb, cu_bool_t, void *))
+{
+    while (node) {
+	if (!cuconP_rbnode_rev_conj_ptr(node->right, cb))
+	    return cu_false;
+	if (!cu_call(cb, *(void **)CU_ALIGNED_PTR_END(node)))
+	    return cu_false;
+	node = node->left;
+    }
+    return cu_true;
+}
+cu_bool_t
+cucon_rbtree_rev_conj_ptr(cucon_rbtree_t tree, cu_clop(cb, cu_bool_t, void *))
+{
+    return cuconP_rbnode_rev_conj_ptr(tree->root, cb);
+}
