@@ -1,3 +1,20 @@
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2007  Petter Urkedal <urkedal@nbi.dk>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 /* Test constructive version of insert */
 
 #include <cucon/rbtree.h>
@@ -28,11 +45,11 @@ insert_some(cucon_rbtree_t tree_ctive, cucon_rbtree_t tree_dtive)
 	int *n = cu_gnew(int);
 	*n = lrand48() % I;
 	t_dtive -= clock();
-	b0 = cucon_rbtree_insert_ptr(tree_dtive,
+	b0 = cucon_rbtree_insert2p_ptr(tree_dtive,
 			(cu_clop(, int, void *, void *))intptr_cmp, &n);
 	t_dtive += clock();
 	t_ctive -= clock();
-	b1 = cucon_rbtree_insert_ptr_ctive(tree_ctive,
+	b1 = cucon_rbtree_cinsert2p_ptr(tree_ctive,
 			(cu_clop(, int, void *, void *))intptr_cmp, &n);
 	t_ctive += clock();
 	assert(b0 == b1);
