@@ -1,3 +1,20 @@
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2007  Petter Urkedal <urkedal@nbi.dk>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include <cucon/pmap.h>
 #include <cucon/ucmap.h>
 #include <time.h>
@@ -36,7 +53,7 @@ bench(cu_bool_t enable_umap, cu_bool_t enable_ucmap)
 	    int j = lrand48() % MOD;
 	    if (enable_ucmap) {
 		ubt_tfind -= clock();
-		cucon_ucmap_find(tree, j);
+		cucon_ucmap_find_ptr(tree, j);
 		ubt_tfind += clock();
 	    }
 	    if (enable_umap) {
