@@ -1,19 +1,18 @@
-/*-*-C-*-
- * Copyright 2002--2005  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2002--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* XXX Loop detection and handling is not debugged.
@@ -24,21 +23,60 @@
 #include <cuex/ex.h>
 #include <cuex/opn.h>
 #include <cuex/algo.h>
-#include <cuex/opr.h>
-#include <cuex/aci.h>
+#include <cuex/oprdefs.h>
 #include <cuex/subst.h>
+#include <cuex/compound.h>
+#include <cuex/intf.h>
 #include <cu/memory.h>
 #include <cu/idr.h>
+#include <cu/int.h>
+#include <cu/ptr_seq.h>
 #include <cucon/list.h>
 #include <cucon/pmap.h>
 #include <cucon/pset.h>
 #include <string.h>
 #include <assert.h>
-#include <ffi.h>
 
 #undef cuex_binary_inject_left
 
 
+cu_rank_t
+cuex_max_arity(cuex_t e)
+{
+    cuex_meta_t e_meta = cuex_meta(e);
+    if (cuex_meta_is_opr(e_meta)) {
+	cu_rank_t r = cuex_opr_r(e_meta);
+	CUEX_OPN_ITER(e_meta, e, ep, r = cu_uint_max(r, cuex_max_arity(ep)));
+	return r;
+    } else if (cuex_meta_is_type(e_meta)) {
+	cuex_intf_compound_t e_c;
+	cuoo_type_t e_type = cuoo_type_from_meta(e_meta);
+	e_c = cuoo_type_impl_ptr(e_type, CUEX_INTF_COMPOUND);
+	if (e_c) {
+	    int r = 1;
+	    cuex_t ep;
+	    cu_ptr_source_t src = cuex_compound_pref_iter_source(e_c, e);
+	    while ((ep = cu_ptr_source_get(src)))
+		r = cu_uint_max(r, cuex_max_arity(ep));
+	    return r;
+	}
+    }
+    return 0;
+}
+
+int
+cuex_max_depth(cuex_t e)
+{
+    cuex_meta_t meta = cuex_meta(e);
+    if (cuex_meta_is_opr(meta)) {
+	int d = 0;
+	CUEX_OPN_ITER(meta, e, ep, d = cu_uint_max(d, cuex_max_depth(ep)));
+	return d + 1;
+    }
+    else
+	return 0;
+}
+
 cu_bool_t
 cuex_match_pmap(cuex_t patn, cuex_t ex, cucon_pmap_t vm)
 {
@@ -86,26 +124,6 @@ cuex_unify(cuex_t ex0, cuex_t ex1)
     return cuex_null();
 }
 
-static cuex_t
-fallback_tran_aci(cuex_meta_t join, cuex_opn_t idy, cuex_opn_t e,
-		  cuex_opn_t acc, cu_clop(cb, cuex_t, cuex_t e))
-{
-    /* Basically a specialised version of "aci.c":aci_tran, for efficiency
-     * since cuex_fallback_tran is very general. */
-    while (e != idy) {
-        cuex_opn_t v;
-        cu_debug_assert(cuex_meta(e) == join);
-        acc = fallback_tran_aci(join, idy, cuex_aci_left(e), acc, cb);
-        if (!acc)
-            return NULL;
-        v = cu_call(cb, cuex_aci_rebranch(e, idy, idy));
-        if (!v)
-            return NULL;
-        acc = cuex_aci_join(join, acc, v);
-        e = cuex_aci_right(e);
-    }
-    return acc;
-}
 cuex_t
 cuex_fallback_tran(cuex_t e, cu_clop(cb, cuex_t, cuex_t))
 {
@@ -114,14 +132,8 @@ cuex_fallback_tran(cuex_t e, cu_clop(cb, cuex_t, cuex_t))
     if (r)
 	return r;
     meta = cuex_meta(e);
-    if (cuex_meta_is_opr(meta)) {
-	if (cuex_opr_is_aci(meta)) {
-	    cuex_opn_t idy = cuex_aci_identity(meta);
-	    return fallback_tran_aci(meta, idy, e, idy, cb);
-	}
-	else
-	    CUEX_OPN_TRAN(meta, e, se, cuex_fallback_tran(se, cb));
-    }
+    if (cuex_meta_is_opr(meta))
+	CUEX_OPN_TRAN(meta, e, se, cuex_fallback_tran(se, cb));
     return e;
 }
 
@@ -176,27 +188,12 @@ cuex_depthout_conj(cuex_t ex, cu_clop(cb, cu_bool_t, cuex_t))
     return cu_call(cb, ex);
 }
 
-cu_clos_def(depthout_tran_aci_cb,
-	    cuex_opn_t cu_proto(cuex_opn_t opn),
-    ( cu_clop(cb, cuex_t, cuex_t); ))
-{
-    cu_clos_self(depthout_tran_aci_cb);
-    return cuex_depthout_tran(opn, self->cb);
-}
 cuex_t
 cuex_depthout_tran(cuex_t ex, cu_clop(cb, cuex_t, cuex_t))
 {
     cuex_meta_t meta = cuex_meta(ex);
-    if (cuex_meta_is_opr(meta)) {
-	if (cuex_opr_is_aci(meta)) {
-	    depthout_tran_aci_cb_t aci_cb;
-	    cu_clos_cct(&aci_cb, depthout_tran_aci_cb);
-	    aci_cb.cb = cb;
-	    ex = cuex_aci_tran(meta, ex, cu_clos_ref(&aci_cb));
-	}
-	else
-	    CUEX_OPN_TRAN(meta, ex, subex, cuex_depthout_tran(subex, cb));
-    }
+    if (cuex_meta_is_opr(meta))
+	CUEX_OPN_TRAN(meta, ex, subex, cuex_depthout_tran(subex, cb));
     return cu_call(cb, ex);
 }
 
@@ -290,11 +287,11 @@ cuex_maxtrees_of_leaftest_tran(cuex_t ex,
 	return ex;
 }
 
-cu_clos_def(cuex_lgr_block_cb,
-	    cu_bool_t cu_proto(cuex_var_t var),
+cu_clos_def(_lgr_block_cb,
+	    cu_prot(cu_bool_t, cuex_var_t var),
 	    (cuex_subst_t subst;))
 {
-    cu_clos_self(cuex_lgr_block_cb);
+    cu_clos_self(_lgr_block_cb);
     cuex_subst_t subst = self->subst;
     cuex_meta_t meta = cuex_meta(var);
     if (cuex_subst_is_active_varmeta(subst, meta)
@@ -306,10 +303,9 @@ cu_clos_def(cuex_lgr_block_cb,
 cu_bool_t
 cuex_lgr(cuex_t ex0, cuex_t ex1, cuex_subst_t subst)
 {
-    cuex_lgr_block_cb_t cb;
-    cu_clos_cct(&cb, cuex_lgr_block_cb);
+    _lgr_block_cb_t cb;
     cb.subst = subst;
-    cuex_depth_conj_vars(ex0, cu_clos_ref(&cb));
+    cuex_depth_conj_vars(ex0, _lgr_block_cb_prep(&cb));
     return cuex_subst_unify(subst, ex1, ex0);
 }
 
@@ -365,7 +361,7 @@ cuex_msg_unify_by_arr(size_t cnt, cuex_t *arr, cu_clop(f, cuex_t, cuex_t *arr))
 }
 
 size_t
-cuex_count_nodes_sans(cuex_t ex, cucon_pmap_t cache)
+cuex_count_unique_nodes_except(cuex_t ex, cucon_pmap_t cache)
 {
     void *ignore;
     if (!cucon_pmap_insert_mem(cache, ex, 0, &ignore))
@@ -375,7 +371,7 @@ cuex_count_nodes_sans(cuex_t ex, cucon_pmap_t cache)
 	size_t i;
 	size_t n = cuex_opn_r(cuex_opn_from_ex(ex));
 	for (i = 0; i < n; ++i)
-	    count += cuex_count_nodes_sans(
+	    count += cuex_count_unique_nodes_except(
 		cuex_opn_at(cuex_opn_from_ex(ex), i), cache);
 	return count;
     }
@@ -384,26 +380,25 @@ cuex_count_nodes_sans(cuex_t ex, cucon_pmap_t cache)
 }
 
 size_t
-cuex_count_nodes(cuex_t ex)
+cuex_count_unique_nodes(cuex_t ex)
 {
     cucon_pmap_t cache = cucon_pmap_new();
-    return cuex_count_nodes_sans(ex, cache);
+    return cuex_count_unique_nodes_except(ex, cache);
 }
 
-cu_clos_def(cuexP_contains_var_in_pmap_h0,
-	    cu_bool_t cu_proto(cuex_var_t var),
+cu_clos_def(_contains_var_in_pmap_cb,
+	    cu_prot(cu_bool_t, cuex_var_t var),
 	    (cucon_pmap_t pmap;))
 {
-    cu_clos_self(cuexP_contains_var_in_pmap_h0);
+    cu_clos_self(_contains_var_in_pmap_cb);
     return !cucon_pmap_find_mem(self->pmap, var);
 }
 cu_bool_t
 cuex_contains_var_in_pmap(cuex_t ex, cucon_pmap_t pmap)
 {
-    cuexP_contains_var_in_pmap_h0_t cb;
-    cu_clos_cct(&cb, cuexP_contains_var_in_pmap_h0);
+    _contains_var_in_pmap_cb_t cb;
     cb.pmap = pmap;
-    return !cuex_depth_conj_vars(ex, cu_clos_ref(&cb));
+    return !cuex_depth_conj_vars(ex, _contains_var_in_pmap_cb_prep(&cb));
 }
 
 
@@ -595,133 +590,31 @@ cuex_binary_inject_left_D(cuex_meta_t opr, cuex_t ex, cuex_t lhs)
     return cuex_binary_inject_left(opr, ex, lhs);
 }
 
-static cu_bool_t
-free_vars_conj(cuex_t ex, cuex_qcset_t qcset, cucon_pset_t excl,
-	       cu_clop(fn, cu_bool_t, cuex_var_t, cucon_pset_t))
-{
-    cuex_meta_t meta = cuex_meta(ex);
-    if (cuex_meta_is_opr(meta)) {
-	if (cuex_opr_is_scoping(meta)) {
-	    size_t i, r = cuex_opr_r(meta);
-	    cuex_t var = cuex_opn_at(ex, 0);
-	    cu_bool_t nonshadow;
-	    cu_debug_assert(cuex_opr_r(meta) > 0);
-	    nonshadow = cucon_pset_insert(excl, var);
-	    for (i = 1; i < r; ++i)
-		if (!free_vars_conj(cuex_opn_at(ex, i), qcset, excl, fn)) {
-		    if (nonshadow)
-			cucon_pset_erase(excl, var);
-		    return cu_false;
-		}
-	    if (nonshadow)
-		cucon_pset_erase(excl, var);
-	}
-	else
-	    CUEX_OPN_CONJ_RETURN(meta, ex, subex,
-		    free_vars_conj(subex, qcset, excl, fn));
-    }
-    else if (cuex_is_varmeta(meta) &&
-	     cuex_qcset_contains(qcset, cuex_varmeta_qcode(meta))) {
-	if (!cucon_pset_find(excl, ex))
-	    return cu_call(fn, ex, excl);
-    }
-    return cu_true;
-}
-
-cu_bool_t
-cuex_free_vars_conj(cuex_t ex, cuex_qcset_t qcset, cucon_pset_t excl,
-		    cu_clop(fn, cu_bool_t, cuex_var_t, cucon_pset_t))
-{
-    if (!excl) {
-	excl = cu_salloc(sizeof(struct cucon_pmap_s));
-	cucon_pset_cct(excl);
-    }
-    return free_vars_conj(ex, qcset, excl, fn);
-}
-
-cu_clos_define_e(cuex_pset_curried_insert_ex,
-	cu_bool_t cu_proto(cuex_t var))
-{
-    cu_clos_self(cuex_pset_curried_insert_ex);
-    cucon_pset_insert(self->accu, var);
-    return cu_true;
-}
-
-cu_clos_define_e(cuex_pset_curried_erase_ex,
-	cu_bool_t cu_proto(cuex_t var))
-{
-    cu_clos_self(cuex_pset_curried_erase_ex);
-    cucon_pset_erase(self->accu, var);
-    return cu_true;
-}
-
-void
-cuex_free_vars_insert(cuex_t e, cuex_qcset_t qcset, cucon_pset_t excl,
-		      cucon_pset_t accu)
-{
-    cuex_pset_curried_insert_ex_t cb;
-    cu_clos_cct(&cb, cuex_pset_curried_insert_ex);
-    cb.accu = accu;
-    cuex_free_vars_conj(e, qcset, excl,
-	cu_clos_ref_cast(cu_bool_t cu_proto(cuex_var_t, cucon_pset_t), &cb));
-}
-
-void
-cuex_free_vars_erase(cuex_t e, cuex_qcset_t qcset, cucon_pset_t excl,
-		     cucon_pset_t accu)
-{
-    cuex_pset_curried_erase_ex_t cb;
-    cu_clos_cct(&cb, cuex_pset_curried_erase_ex);
-    cb.accu = accu;
-    cuex_free_vars_conj(e, qcset, excl,
-	cu_clos_ref_cast(cu_bool_t cu_proto(cuex_var_t, cucon_pset_t), &cb));
-}
-
-cu_clos_def(free_vars_count_cb,
-	    cu_bool_t cu_proto(cuex_var_t var, cucon_pset_t bound),
-	( int cnt; ))
-{
-    cu_clos_self(free_vars_count_cb);
-    ++self->cnt;
-    return cu_true;
-}
-
-int
-cuex_free_vars_count(cuex_t e, cuex_qcset_t qcset, cucon_pset_t excl)
-{
-    free_vars_count_cb_t cb;
-    cu_clos_cct(&cb, free_vars_count_cb);
-    cb.cnt = 0;
-    cuex_free_vars_conj(e, qcset, excl, cu_clos_ref(&cb));
-    return cb.cnt;
-}
-
-cu_clos_def(outmost_quantify_vars_cb,
-	    void cu_proto(void const *var),
+cu_clos_def(_outmost_quantify_vars_cb,
+	    cu_prot(void, void const *var),
 	( cuex_t e;
 	  cuex_meta_t opr; ))
 {
-    cu_clos_self(outmost_quantify_vars_cb);
+    cu_clos_self(_outmost_quantify_vars_cb);
     self->e = cuex_opn(self->opr, var, self->e);
 }
 
 cuex_t
 cuex_outmost_quantify_vars(cuex_meta_t opr, cucon_pset_t vars, cuex_t e)
 {
-    outmost_quantify_vars_cb_t cb;
+    _outmost_quantify_vars_cb_t cb;
     cu_debug_assert(cuex_opr_r(opr) == 2);
-    cu_clos_cct(&cb, outmost_quantify_vars_cb);
     cb.e = e;
     cb.opr = opr;
-    cucon_pset_iter(vars, cu_clos_ref(&cb));
+    cucon_pset_iter(vars, _outmost_quantify_vars_cb_prep(&cb));
     return cb.e;
 }
 
-cu_clos_def(autoquantify_uvw_xyz_cb,
-	    cuex_t cu_proto(cuex_t ex),
+cu_clos_def(_autoquantify_uvw_xyz_cb,
+	    cu_prot(cuex_t, cuex_t ex),
 	    (cucon_pmap_t env;))
 {
-    cu_clos_self(autoquantify_uvw_xyz_cb);
+    cu_clos_self(_autoquantify_uvw_xyz_cb);
     assert(ex);
     if (cuex_is_idr(ex)) {
 	cuex_pvar_t *var;
@@ -745,17 +638,16 @@ cu_clos_def(autoquantify_uvw_xyz_cb,
 cuex_t
 cuex_autoquantify_uvw_xyz(cuex_t ex, cucon_pmap_t env)
 {
-    autoquantify_uvw_xyz_cb_t cb;
-    cu_clos_cct(&cb, autoquantify_uvw_xyz_cb);
+    _autoquantify_uvw_xyz_cb_t cb;
     cb.env = env;
-    return cuex_depthout_tran(ex, cu_clos_ref(&cb));
+    return cuex_depthout_tran(ex, _autoquantify_uvw_xyz_cb_prep(&cb));
 }
 
-cu_clos_def(ex_stats_cb,
-	    cu_bool_t cu_proto(cuex_t ex),
+cu_clos_def(_ex_stats_cb,
+	    cu_prot(cu_bool_t, cuex_t ex),
 	    (cuex_stats_t *stats;))
 {
-    cu_clos_self(ex_stats_cb);
+    cu_clos_self(_ex_stats_cb);
     cuex_meta_t meta = cuex_meta(ex);
     switch (cuex_meta_kind(meta)) {
 	case cuex_meta_kind_type:
@@ -790,11 +682,10 @@ cu_clos_def(ex_stats_cb,
 void
 cuex_stats(cuex_t ex, cuex_stats_t *stats)
 {
-    ex_stats_cb_t cb;
-    cu_clos_cct(&cb, ex_stats_cb);
+    _ex_stats_cb_t cb;
     cb.stats = stats;
     memset(stats, 0, sizeof(cuex_stats_t));
-    cuex_depthout_conj(ex, cu_clos_ref(&cb));
+    cuex_depthout_conj(ex, _ex_stats_cb_prep(&cb));
     stats->var_cnt
 	= stats->strong_var_cnt
 	+ stats->weak_var_cnt
