@@ -1,23 +1,23 @@
-/*-*-C-*-
- * Copyright 2005  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2005--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <cuex/ex.h>
 #include <cuex/opn.h>
+#include <cuex/oprdefs.h>
 #include <cu/sref.h>
 
 cuex_t cuexP_null;
@@ -27,5 +27,5 @@ void
 cuexP_ex_init()
 {
     cuexP_null = cuex_opn(cuex_opr_null);
-    cuexP_unknown = cuex_opn(cuex_opr_unknown_0);
+    cuexP_unknown = cuex_opn(CUEX_O0_UNKNOWN);
 }
