@@ -1,19 +1,18 @@
-/*-*-C-*-
- * Copyright 2005  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2005--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <cuex/opntpl.h>
@@ -23,7 +22,7 @@ cuex_opntpl_new(cuex_meta_t opr)
 {
     cu_rank_t r = cuex_opr_r(opr);
     size_t size = cuex_sizeof_opn(r);
-    cuex_opn_t opn = cuex_oalloc(opr, size);
+    cuex_opn_t opn = cuexP_oalloc(opr, size);
     return opn;
 }
 
