@@ -1,22 +1,21 @@
-/*-*-C-*-
- * Copyright 2003--2004  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2003--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define CHI_SSFN_C
+#define CUEX_SSFN_C
 #include <cu/idr.h>
 #include <cu/memory.h>
 #include <cu/debug.h>
@@ -30,18 +29,14 @@
 #include <cucon/slink.h>
 #include <cucon/rbtree.h>
 #include <cucon/list.h>
-#include <cudyn/properties.h>
+#include <cuoo/properties.h>
 #include <string.h>
 #include <assert.h>
 
-#define CHI_SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU 1
+cu_dlog_def(_file, "dtag=cuex.ssfn");
+
+#define SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU 1
 
-/* #define CHI_SSFN_ENABLE_VERBOSE_DEBUG */
-#ifdef CHI_SSFN_ENABLE_VERBOSE_DEBUG
-#  define D_PRINTF(args...) cu_fprintf(stderr, args)
-#else
-#  define D_PRINTF(args...) ((void)0)
-#endif
 
 /* Helpers
  * ------- */
@@ -118,8 +113,8 @@ cuexP_sctr_lgr(int *lhs, int *rhs)
 static void
 cuex_ssfn_node_cct(cuex_ssfn_node_t node, cuex_ssfn_seqno_t seqno)
 {
-    cucon_pmap_cct(&node->match_exaddr);
-    cucon_umap_cct(&node->match_opr);
+    cucon_pmap_init(&node->match_exaddr);
+    cucon_umap_init(&node->match_opr);
     node->var_cntn = NULL;
     node->var_qcode = cuex_qcode_na;
     node->seqno = seqno;
@@ -128,13 +123,13 @@ cuex_ssfn_node_cct(cuex_ssfn_node_t node, cuex_ssfn_seqno_t seqno)
 void
 cuex_ssfn_cct(cuex_ssfn_t ssfn)
 {
-    cuex_ssfn_node_cct(cu_upcast(cuex_ssfn_node_s, ssfn), 0);
+    cuex_ssfn_node_cct(cu_to(cuex_ssfn_node, ssfn), 0);
 }
 
 cuex_ssfn_t
 cuex_ssfn_new(void)
 {
-    cuex_ssfn_t ssfn = cu_galloc(sizeof(struct cuex_ssfn_s));
+    cuex_ssfn_t ssfn = cu_galloc(sizeof(struct cuex_ssfn));
     cuex_ssfn_cct(ssfn);
     return ssfn;
 }
@@ -142,7 +137,7 @@ cuex_ssfn_new(void)
 void
 cuex_ssfn_dct(cuex_ssfn_t ssfn)
 {
-    memset(ssfn, 0, sizeof(struct cuex_ssfn_s));
+    memset(ssfn, 0, sizeof(struct cuex_ssfn));
 }
 
 void
@@ -158,7 +153,7 @@ cuex_ssfn_node_t
 cuex_ssfn_node_new_sub(cu_count_t arg_cnt, cuex_ssfn_seqno_t seqno)
 {
     cuex_ssfn_node_t node
-	= cu_galloc(CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
+	= cu_galloc(CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
 		     + sizeof(cuex_ssfn_node_t)*arg_cnt);
     cuex_ssfn_node_cct(node, seqno);
     while (arg_cnt > 0)
@@ -169,51 +164,51 @@ cuex_ssfn_node_new_sub(cu_count_t arg_cnt, cuex_ssfn_seqno_t seqno)
 /* Copy and Clone
  * -------------- */
 
-typedef struct cuexP_ssfn_cct_copy_jargs_s
+typedef struct _ssfn_init_copy_jargs
 {
     size_t slot_size;
     cu_count_t var_cnt;
     cu_count_t pending_cnt;
     cu_clop(value_cct_copy, void, void *, void *);
-} *cuexP_ssfn_cct_copy_jargs_t;
+} *_ssfn_init_copy_jargs_t;
 
-static void cuexP_ssfn_cct_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
-			       cuexP_ssfn_cct_copy_jargs_t jargs);
+static void _ssfn_init_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
+			    _ssfn_init_copy_jargs_t jargs);
 
-cu_clos_def(cuexP_ssfn_cct_copy_idr_or_exaddr,
-	    void cu_proto(void *dst_slot, void *src_slot, uintptr_t key),
-	    (cuexP_ssfn_cct_copy_jargs_t jargs;))
+cu_clos_def(_ssfn_init_copy_idr_or_exaddr,
+	    cu_prot(void, void *dst_slot, void *src_slot, uintptr_t key),
+	    (_ssfn_init_copy_jargs_t jargs;))
 {
-    cu_clos_self(cuexP_ssfn_cct_copy_idr_or_exaddr);
-    cuexP_ssfn_cct_copy(dst_slot, src_slot, self->jargs);
+    cu_clos_self(_ssfn_init_copy_idr_or_exaddr);
+    _ssfn_init_copy(dst_slot, src_slot, self->jargs);
 }
-cu_clos_def(cuexP_ssfn_cct_copy_opr,
-	    cucon_pmap_node_t cu_proto(void *src_slot, uintptr_t key),
-	    (cuexP_ssfn_cct_copy_jargs_t jargs;))
+cu_clos_def(_ssfn_init_copy_opr,
+	    cu_prot(cucon_umap_node_t, void *src_slot, uintptr_t key),
+	    (_ssfn_init_copy_jargs_t jargs;))
 {
-    cu_clos_self(cuexP_ssfn_cct_copy_opr);
-    cucon_pmap_node_t node;
-    cuexP_ssfn_cct_copy_jargs_t jargs = self->jargs;
+    cu_clos_self(_ssfn_init_copy_opr);
+    cucon_umap_node_t node;
+    _ssfn_init_copy_jargs_t jargs = self->jargs;
     cu_count_t pending_cnt_save = jargs->pending_cnt;
     jargs->pending_cnt += cuex_opr_r(key);
     if (jargs->pending_cnt == 0) {
-	node = cucon_pmap_node_alloc(jargs->slot_size);
+	node = cucon_umap_node_alloc(jargs->slot_size);
 	cu_call(jargs->value_cct_copy,
-		 cucon_pmap_node_get_mem(node), src_slot);
+		cucon_umap_node_get_mem(node), src_slot);
     }
     else {
 	cu_count_t var_cnt = jargs->var_cnt;
-	node = cucon_pmap_node_alloc(CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
-				   + sizeof(cuex_ssfn_node_t)*var_cnt);
-	cuexP_ssfn_cct_copy(cucon_pmap_node_get_mem(node), src_slot, jargs);
+	node = cucon_umap_node_alloc(CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
+				     + sizeof(cuex_ssfn_node_t)*var_cnt);
+	_ssfn_init_copy(cucon_umap_node_get_mem(node), src_slot, jargs);
     }
     jargs->pending_cnt = pending_cnt_save;
     return node;
 }
 
 static void
-cuexP_ssfn_cct_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
-		     cuexP_ssfn_cct_copy_jargs_t jargs)
+_ssfn_init_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
+		_ssfn_init_copy_jargs_t jargs)
 {
     if (jargs->pending_cnt == 0)
 	cu_call(jargs->value_cct_copy, (void*)dst, (void*)src);
@@ -222,13 +217,13 @@ cuexP_ssfn_cct_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
 	cu_count_t i;
 	size_t sub_size;
 	size_t sub_size_var_cntn;
-	cuexP_ssfn_cct_copy_idr_or_exaddr_t idr_or_exaddr_cb;
-	cuexP_ssfn_cct_copy_opr_t opr_cb;
+	_ssfn_init_copy_idr_or_exaddr_t idr_or_exaddr_cb;
+	_ssfn_init_copy_opr_t opr_cb;
 
 	--jargs->pending_cnt;
 	if (jargs->pending_cnt > 0) {
 	    sub_size
-		= CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
+		= CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
 		+ sizeof(cuex_ssfn_node_t)*var_cnt;
 	    sub_size_var_cntn = sub_size + sizeof(cuex_ssfn_node_t);
 	}
@@ -237,22 +232,21 @@ cuexP_ssfn_cct_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
 	    sub_size_var_cntn = sub_size;
 	}
 
-	cu_clos_cct(&idr_or_exaddr_cb, cuexP_ssfn_cct_copy_idr_or_exaddr);
 	idr_or_exaddr_cb.jargs = jargs;
-	cucon_pmap_cct_copy_mem_ctor(&dst->match_exaddr, &src->match_exaddr,
-				     sub_size, cu_clos_ref(&idr_or_exaddr_cb));
+	cucon_pmap_init_copy_mem_ctor(&dst->match_exaddr, &src->match_exaddr,
+		 sub_size,
+		 _ssfn_init_copy_idr_or_exaddr_prep(&idr_or_exaddr_cb));
 
-	cu_clos_cct(&opr_cb, cuexP_ssfn_cct_copy_opr);
 	opr_cb.jargs = jargs;
-	cucon_umap_cct_copy_node(&dst->match_opr, &src->match_opr,
-				 cu_clos_ref(&opr_cb));
+	cucon_umap_init_copy_node(&dst->match_opr, &src->match_opr,
+				  _ssfn_init_copy_opr_prep(&opr_cb));
 
 	dst->var_qcode = src->var_qcode;
 
 	if (src->var_cntn) {
 	    dst->var_cntn = cu_galloc(sub_size_var_cntn);
 	    ++jargs->var_cnt;
-	    cuexP_ssfn_cct_copy(dst->var_cntn, src->var_cntn, jargs);
+	    _ssfn_init_copy(dst->var_cntn, src->var_cntn, jargs);
 	    --jargs->var_cnt;
 	}
 
@@ -261,7 +255,7 @@ cuexP_ssfn_cct_copy(cuex_ssfn_node_t dst, cuex_ssfn_node_t src,
 	    if (src_cntn) {
 		cuex_ssfn_node_t dst_cntn = cu_galloc(sub_size);
 		ssfn_unify_cntn(dst, i) = dst_cntn;
-		cuexP_ssfn_cct_copy(dst_cntn, src_cntn, jargs);
+		_ssfn_init_copy(dst_cntn, src_cntn, jargs);
 	    }
 	    else
 		ssfn_unify_cntn(dst, i) = NULL;
@@ -278,13 +272,13 @@ cuex_ssfn_cct_copy_mem(cuex_ssfn_t dst, cuex_ssfn_t src,
 		       size_t slot_size,
 		       cu_clop(value_cct_copy, void, void *, void *))
 {
-    struct cuexP_ssfn_cct_copy_jargs_s jargs;
+    struct _ssfn_init_copy_jargs jargs;
     jargs.slot_size = slot_size;
     jargs.var_cnt = 0;
     jargs.pending_cnt = 1;
     jargs.value_cct_copy = value_cct_copy;
-    cuexP_ssfn_cct_copy(cu_upcast(cuex_ssfn_node_s, dst),
-		       cu_upcast(cuex_ssfn_node_s, src), &jargs);
+    _ssfn_init_copy(cu_to(cuex_ssfn_node, dst),
+		    cu_to(cuex_ssfn_node, src), &jargs);
 }
 
 cuex_ssfn_t
@@ -292,7 +286,7 @@ cuex_ssfn_new_copy_mem(cuex_ssfn_t src,
 		       size_t slot_size,
 		       cu_clop(value_cct_copy, void, void *, void *))
 {
-    cuex_ssfn_t dst = cu_gnew(struct cuex_ssfn_s);
+    cuex_ssfn_t dst = cu_gnew(struct cuex_ssfn);
     cuex_ssfn_cct_copy_mem(dst, src, slot_size, value_cct_copy);
     return dst;
 }
@@ -396,15 +390,15 @@ void
 cuex_ssfn_find(cuex_ssfn_t ssfn, cuex_t key,
 	       cu_clop(out, cu_bool_t, cu_count_t, cuex_t *, void *))
 {
-    struct cuexP_ssfn_find_jargs_s jargs;
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    struct cuexP_ssfn_find_jargs jargs;
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     jargs.sctr_most_specific = NULL;
     jargs.receiver = out;
     jargs.arg_cnt = 0;
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
-    cuexP_ssfn_find(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find(cu_to(cuex_ssfn_node, ssfn), &jargs);
 }
 
 
@@ -428,7 +422,7 @@ struct cuexP_ssfn_find_it_frame_s
 {
     cuex_ssfn_node_t node;
     cuexP_ssfn_find_it_state_t state;
-    cuex_t ex;
+    cuex_t ex, ex0;
     int i;
 };
 
@@ -436,13 +430,13 @@ void
 cuex_ssfn_find_it_cct(cuex_ssfn_find_it_t find_it, cuex_ssfn_t ssfn, cuex_t key)
 {
     cuexP_ssfn_find_it_frame_t frame;
-    cucon_stack_cct(&find_it->stack);
+    cucon_stack_init(&find_it->stack);
     frame = cucon_stack_alloc(&find_it->stack,
 			    sizeof(struct cuexP_ssfn_find_it_frame_s));
-    frame->node = cu_upcast(cuex_ssfn_node_s, ssfn);
-    cucon_stack_cct(&find_it->jargs.input);
-    cucon_stack_cct(&find_it->jargs.output);
-    cucon_stack_cct(&find_it->jargs.sctr);
+    frame->node = cu_to(cuex_ssfn_node, ssfn);
+    cucon_stack_init(&find_it->jargs.input);
+    cucon_stack_init(&find_it->jargs.output);
+    cucon_stack_init(&find_it->jargs.sctr);
     find_it->jargs.sctr_most_specific = NULL;
     find_it->jargs.arg_cnt = 0;
     CUCON_STACK_PUSH(&find_it->jargs.input, cuex_t, key);
@@ -453,7 +447,7 @@ cuex_ssfn_find_it_cct(cuex_ssfn_find_it_t find_it, cuex_ssfn_t ssfn, cuex_t key)
 cuex_ssfn_find_it_t
 cuex_ssfn_find_it_new(cuex_ssfn_t ssfn, cuex_t key)
 {
-    cuex_ssfn_find_it_t find_it = cu_gnew(struct cuex_ssfn_find_it_s);
+    cuex_ssfn_find_it_t find_it = cu_gnew(struct cuex_ssfn_find_it);
     cuex_ssfn_find_it_cct(find_it, ssfn, key);
     return find_it;
 }
@@ -574,8 +568,7 @@ enter:
     }
     for (i = 0; i < jargs->arg_cnt; ++i)
 	if (ssfn_unify_cntn(node, i)) {
-	    cuex_t *arg_pos
-		= cucon_stack_at(&jargs->output, i*sizeof(cuex_t));
+	    cuex_t *arg_pos = cucon_stack_at(&jargs->output, i*sizeof(cuex_t));
 	    cuex_t ex0 = *arg_pos;
 	    cuex_t ex1 = cuex_unify(ex0, ex);
 	    if (!cuex_is_null(ex1)) {
@@ -587,9 +580,11 @@ enter:
 			    sizeof(struct cuexP_ssfn_find_it_frame_s));
 		frame->node = ssfn_unify_cntn(node, i);
 		frame->i = i;
+		frame->ex0 = ex0;
 		goto enter;
 	    return_in_unify_var:
 		i = frame->i;
+		ex0 = frame->ex0;
 		arg_pos = cucon_stack_at(&jargs->output, i*sizeof(cuex_t));
 		CUCON_STACK_POP(&jargs->sctr, int);
 		*arg_pos = ex0;
@@ -653,18 +648,18 @@ cuex_ssfn_intersection_find_aux(
 {
     cuex_ssfn_find_it_t it_match;
     cuex_ssfn_find_it_t it_spec CU_NOINIT(NULL);
-    cucon_list_it_t it_l;
-    struct cucon_priq_s q_it;
+    cucon_listnode_t it_l;
+    struct cucon_priq q_it;
 
-    cucon_priq_cct(&q_it, (cu_clop(, cu_bool_t, void *, void *))
+    cucon_priq_init(&q_it, (cu_clop(, cu_bool_t, void *, void *))
 				cuex_ssfn_find_it_match_lgr_clop);
     it_match = cuex_ssfn_find_it_new(ssfn_first, key);
     if (!cuex_ssfn_find_it_is_end(it_match))
 	cucon_priq_insert(&q_it, it_match);
     for (it_l = cucon_list_begin(ssfn_list);
 	 it_l != cucon_list_end(ssfn_list);
-	 it_l = cucon_list_it_next(it_l)) {
-	cuex_ssfn_t ssfn = cucon_list_it_get_ptr(it_l);
+	 it_l = cucon_listnode_next(it_l)) {
+	cuex_ssfn_t ssfn = cucon_listnode_ptr(it_l);
 	it_match = cuex_ssfn_find_it_new(ssfn, key);
 	if (!cuex_ssfn_find_it_is_end(it_match))
 	    cucon_priq_insert(&q_it, it_match);
@@ -705,11 +700,11 @@ cuex_ssfn_intersection_find_aux(
  * -------------------- */
 
 typedef struct cuexP_ssfn_insert_jargs_s {
-    struct cucon_stack_s input;
+    struct cucon_stack input;
     size_t slot_size;
     void **slot;
     cu_count_t var_cnt;
-    struct cucon_pmap_s var_to_index;	/* cu_var_t → cu_count_t */
+    struct cucon_pmap var_to_index;	/* cu_var_t → cu_count_t */
     cuex_ssfn_seqno_t seqno;
 }				     *cuexP_ssfn_insert_jargs_t;
 static cu_bool_t
@@ -740,7 +735,7 @@ tail_rec:
 	    cuex_ssfn_node_t subnode;
 	    if (cucon_umap_insert_mem(
 		    &node->match_opr, cuex_meta(ex),
-		    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
+		    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
 		    + sizeof(cuex_ssfn_node_t)*jargs->var_cnt,
 		    (void *)&subnode))
 		cuex_ssfn_node_cct_sub(subnode, jargs->var_cnt, jargs->seqno);
@@ -757,7 +752,7 @@ tail_rec:
 	    cuex_ssfn_node_t subnode;
 	    if (cucon_pmap_insert_mem(
 		    &node->match_exaddr, ex,
-		    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
+		    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
 		    + sizeof(cuex_ssfn_node_t)*jargs->var_cnt,
 		    (void *)&subnode))
 		cuex_ssfn_node_cct_sub(subnode, jargs->var_cnt,
@@ -811,7 +806,7 @@ tail_rec:
 		    cuex_ssfn_node_t subnode;
 		    if (cucon_pmap_insert_mem(
 			    &node->match_exaddr, ex,
-			    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node_s)
+			    CU_ALIGNED_SIZEOF(struct cuex_ssfn_node)
 			    + sizeof(cuex_ssfn_node_t)*jargs->var_cnt,
 			    (void *)&subnode))
 			cuex_ssfn_node_cct_sub(subnode, jargs->var_cnt,
@@ -821,7 +816,7 @@ tail_rec:
 		}
 
 	    default:
-		cu_errf_at(cudyn_sref(ex),
+		cu_errf_at(cuoo_sref(ex),
 			   "Badly quantified variable %! for pattern.", ex);
 		cu_debug_unreachable();
 		/* XXX at least until there are useful cases which can
@@ -830,7 +825,7 @@ tail_rec:
 	    }
 	}
 	else {
-	    cu_errf_at(cudyn_sref(ex), "‘%!’ is invaild in pattern.", ex);
+	    cu_errf_at(cuoo_sref(ex), "‘%!’ is invaild in pattern.", ex);
 	    return cu_false;
 	}
     }
@@ -847,13 +842,13 @@ cuex_ssfn_insert_mem(cuex_ssfn_t ssfn, cuex_t patn,
     struct cuexP_ssfn_insert_jargs_s jargs;
     cu_bool_t st;
     cu_debug_assert(ssfn && patn);
-    cucon_stack_cct(&jargs.input);
+    cucon_stack_init(&jargs.input);
     jargs.slot_size = slot_size;
     jargs.slot = slot_out;
     jargs.var_cnt = 0;
-    cucon_pmap_cct(&jargs.var_to_index);
+    cucon_pmap_init(&jargs.var_to_index);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, patn);
-    st = cuexP_ssfn_insert_mem(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    st = cuexP_ssfn_insert_mem(cu_to(cuex_ssfn_node, ssfn), &jargs);
     n = cucon_pmap_size(&jargs.var_to_index);
     if (var_cnt_out)
 	*var_cnt_out = n;
@@ -875,10 +870,10 @@ cuex_ssfn_insert_mem(cuex_ssfn_t ssfn, cuex_t patn,
 
 typedef struct cuexP_ssfn_find_mgu_jargs_s
 {
-    struct cucon_stack_s targets;
-    struct cucon_stack_s input;
-    struct cucon_stack_s output;
-    struct cucon_stack_s sctr;
+    struct cucon_stack targets;
+    struct cucon_stack input;
+    struct cucon_stack output;
+    struct cucon_stack sctr;
     cuex_ssfn_find_cb_t cb;
     cuex_var_t key_var;
     cuex_t key_arg;
@@ -888,7 +883,7 @@ typedef struct cuexP_ssfn_find_mgu_jargs_s
     cu_bool_t do_lgr_w;
     cu_bool_t do_mgr;
     cu_bool_t do_mgr_w;
-    struct cuex_ssfn_matchinfo_s minfo;
+    struct cuex_ssfn_matchinfo minfo;
     cuex_ssfn_ctrl_t ctrl;
     cuex_ssfn_seqno_t min_seqno;
 } *cuexP_ssfn_find_mgu_jargs_t;
@@ -925,15 +920,15 @@ cuexP_ssfn_find_mgu_maybe_expand(cuex_ssfn_node_t node,
 static cu_bool_t
 cuexP_ssfn_find_mgu(cuex_ssfn_node_t node, cuexP_ssfn_find_mgu_jargs_t jargs);
 
-cu_clos_def(cuexP_ssfn_find_mgu_exaddr,
-	    cu_bool_t cu_proto(void const *key, void *node),
+cu_clos_def(_ssfn_find_mgu_exaddr,
+	    cu_prot(cu_bool_t, void const *key, void *node),
 	    ( cuex_t *target;
 	      cuexP_ssfn_find_mgu_jargs_t jargs;
 	      cucon_slink_t deletable; ))
 {
 #define key ((cuex_t)key)
 #define node ((cuex_ssfn_node_t)node)
-    cu_clos_self(cuexP_ssfn_find_mgu_exaddr);
+    cu_clos_self(_ssfn_find_mgu_exaddr);
     cu_bool_t do_cont;
     if (self->target)
 	*self->target = key;
@@ -946,14 +941,14 @@ cu_clos_def(cuexP_ssfn_find_mgu_exaddr,
 #undef key
 #undef node
 }
-cu_clos_def(cuexP_ssfn_find_mgu_opr,
-	    cu_bool_t cu_proto(uintptr_t key, void *node),
+cu_clos_def(_ssfn_find_mgu_opr,
+	    cu_prot(cu_bool_t, uintptr_t key, void *node),
 	    ( cuex_t *target;
 	      cuexP_ssfn_find_mgu_jargs_t jargs;
 	      cucon_slink_t deletable; ))
 {
 #define node ((cuex_ssfn_node_t)node)
-    cu_clos_self(cuexP_ssfn_find_mgu_opr);
+    cu_clos_self(_ssfn_find_mgu_opr);
     cu_bool_t do_cont;
     cu_rank_t n = cuex_opr_r(key);
     cu_rank_t i = n;
@@ -1019,13 +1014,12 @@ cuexP_ssfn_find_mgu_expand(cuex_ssfn_node_t node, cuex_t *target,
     cu_bool_t have_deletion = cu_false;
     cu_bool_t do_continue = cu_true;
     {
-	cuexP_ssfn_find_mgu_exaddr_t cb;
-	cu_clos_cct(&cb, cuexP_ssfn_find_mgu_exaddr);
+	_ssfn_find_mgu_exaddr_t cb;
 	cb.target = target;
 	cb.jargs = jargs;
 	cb.deletable = cucon_slink_empty();
 	do_continue = cucon_pmap_conj_mem(&node->match_exaddr,
-					  cu_clos_ref(&cb));
+					  _ssfn_find_mgu_exaddr_prep(&cb));
 	if (!cucon_slink_is_empty(cb.deletable)) {
 	    have_deletion = cu_true;
 	    do {
@@ -1038,12 +1032,12 @@ cuexP_ssfn_find_mgu_expand(cuex_ssfn_node_t node, cuex_t *target,
 	    goto done;
     }
     {
-	cuexP_ssfn_find_mgu_opr_t cb;
-	cu_clos_cct(&cb, cuexP_ssfn_find_mgu_opr);
+	_ssfn_find_mgu_opr_t cb;
 	cb.target = target;
 	cb.jargs = jargs;
 	cb.deletable = cucon_slink_empty();
-	do_continue = cucon_umap_conj_mem(&node->match_opr, cu_clos_ref(&cb));
+	do_continue = cucon_umap_conj_mem(&node->match_opr,
+					  _ssfn_find_mgu_opr_prep(&cb));
 	if (!cucon_slink_is_empty(cb.deletable)) {
 	    have_deletion = cu_true;
 	    do {
@@ -1079,7 +1073,7 @@ cuexP_ssfn_find_mgu_expand(cuex_ssfn_node_t node, cuex_t *target,
     if (node->var_cntn) {
 	cuex_var_t var;
 	cuex_subst_t subst_save;
-#ifdef CHI_SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU
+#ifdef SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU
 	if (jargs->do_mgr && node->var_qcode == cuex_qcode_active_s
 	    && target == &jargs->key_arg)
 	    goto done;
@@ -1110,6 +1104,7 @@ done:
     return do_continue;
 }
 
+#if 0
 static cu_rank_t
 cuexP_count_active_vars(cuex_t ex)
 {
@@ -1137,6 +1132,7 @@ recurse:
     }
     return sum;
 }
+#endif
 
 static cu_bool_t
 cuexP_ssfn_find_mgu(cuex_ssfn_node_t node, cuexP_ssfn_find_mgu_jargs_t jargs)
@@ -1281,8 +1277,8 @@ cuexP_ssfn_find_mgu(cuex_ssfn_node_t node, cuexP_ssfn_find_mgu_jargs_t jargs)
 			|| cuex_var_qcode(cuex_var_from_ex(*arg_pos))
 			== cuex_qcode_active_s)) {
 		    jargs->subst = cuex_subst_new_uw_clone(subst_save);
-		    cu_dprintf("ssfn", "Unify for pattern var, %! = %!?\n",
-			       *arg_pos, key);
+		    cu_dlogf(_file, "Unify for pattern var, %! = %!?",
+			     *arg_pos, key);
 		    if (cuex_subst_unify(jargs->subst, *arg_pos, key)) {
 			CUCON_STACK_PUSH(&jargs->sctr, int,
 					 CHIB_SCTR_SPECIFIC);
@@ -1293,7 +1289,7 @@ cuexP_ssfn_find_mgu(cuex_ssfn_node_t node, cuexP_ssfn_find_mgu_jargs_t jargs)
 		    }
 		}
 		else {
-		    cu_dprintf("ssfn", "Compare %! = %!?\n", *arg_pos, key);
+		    cu_dlogf(_file, "Compare %! = %!?", *arg_pos, key);
 		    if (cuex_eq(*arg_pos, key)) {
 			CUCON_STACK_PUSH(&jargs->sctr, int,
 					 CHIB_SCTR_SPECIFIC);
@@ -1308,7 +1304,7 @@ cuexP_ssfn_find_mgu(cuex_ssfn_node_t node, cuexP_ssfn_find_mgu_jargs_t jargs)
 		    cuexP_ssfn_node_check_deletable(node, jargs);
 		}
 	    }
-#ifdef CHI_SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU
+#ifdef SSFN_TUNING_AVOID_CROSS_SUBST_IN_MGU
 	if (jargs->do_lgr && node->var_qcode == cuex_qcode_active_w
 	    && ( cuex_is_varmeta_q(key_meta, cuex_qcode_u) ||
 		 cuex_is_varmeta_q(key_meta, cuex_qcode_w) )
@@ -1369,10 +1365,10 @@ cuex_ssfn_find_mgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 		   cuex_ssfn_find_cb_t cb)
 {
     struct cuexP_ssfn_find_mgu_jargs_s jargs;
-    cucon_stack_cct(&jargs.targets);
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    cucon_stack_init(&jargs.targets);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.cb = cb;
     jargs.subst = cuex_subst_new_uw_clone(subst);
@@ -1384,7 +1380,7 @@ cuex_ssfn_find_mgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
     jargs.sctr_most_specific = NULL;
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.min_seqno = min_seqno;
-    cuexP_ssfn_find_mgu(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find_mgu(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1394,10 +1390,10 @@ cuex_ssfn_find_wpmgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 		    cuex_ssfn_find_cb_t cb)
 {
     struct cuexP_ssfn_find_mgu_jargs_s jargs;
-    cucon_stack_cct(&jargs.targets);
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    cucon_stack_init(&jargs.targets);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.cb = cb;
     jargs.subst = cuex_subst_new_uw_clone(subst);
@@ -1409,7 +1405,7 @@ cuex_ssfn_find_wpmgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
     jargs.sctr_most_specific = NULL;
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.min_seqno = min_seqno;
-    cuexP_ssfn_find_mgu(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find_mgu(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1419,10 +1415,10 @@ cuex_ssfn_find_lgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 		  cuex_ssfn_find_cb_t cb)
 {
     struct cuexP_ssfn_find_mgu_jargs_s jargs;
-    cucon_stack_cct(&jargs.targets);
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    cucon_stack_init(&jargs.targets);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.cb = cb;
     jargs.subst = cuex_subst_new_uw_clone(subst);
@@ -1434,7 +1430,7 @@ cuex_ssfn_find_lgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
     jargs.sctr_most_specific = NULL;
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.min_seqno = min_seqno;
-    cuexP_ssfn_find_mgu(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find_mgu(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1444,10 +1440,10 @@ cuex_ssfn_find_mgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 		  cuex_ssfn_find_cb_t cb)
 {
     struct cuexP_ssfn_find_mgu_jargs_s jargs;
-    cucon_stack_cct(&jargs.targets);
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    cucon_stack_init(&jargs.targets);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.cb = cb;
     jargs.subst = cuex_subst_new_uw_clone(subst);
@@ -1459,7 +1455,7 @@ cuex_ssfn_find_mgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
     jargs.sctr_most_specific = NULL;
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.min_seqno = min_seqno;
-    cuexP_ssfn_find_mgu(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find_mgu(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1469,10 +1465,10 @@ cuex_ssfn_find_wpmgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 		    cuex_ssfn_find_cb_t cb)
 {
     struct cuexP_ssfn_find_mgu_jargs_s jargs;
-    cucon_stack_cct(&jargs.targets);
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
-    cucon_stack_cct(&jargs.sctr);
+    cucon_stack_init(&jargs.targets);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
+    cucon_stack_init(&jargs.sctr);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.cb = cb;
     jargs.subst = cuex_subst_new_uw_clone(subst);
@@ -1484,7 +1480,7 @@ cuex_ssfn_find_wpmgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
     jargs.sctr_most_specific = NULL;
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.min_seqno = min_seqno;
-    cuexP_ssfn_find_mgu(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cuexP_ssfn_find_mgu(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1495,7 +1491,7 @@ cuex_ssfn_find_wpmgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 
 cu_clop_def(check_strength_cb2, void, cuex_veqv_t veqv)
 {
-    cu_debug_assert(veqv->qcode == cuex_qcode_active_s);
+    cu_debug_assert(cuex_veqv_qcode(veqv) == cuex_qcode_active_s);
 }
 cu_clop_def(check_strength_cb, cuex_ssfn_ctrl_t,
 	    void *slot, cuex_subst_t subst,
@@ -1503,7 +1499,7 @@ cu_clop_def(check_strength_cb, cuex_ssfn_ctrl_t,
 	    cuex_ssfn_matchinfo_t ign)
 {
     cuex_subst_iter_veqv(subst, check_strength_cb2);
-#ifdef CHI_SSFN_ENABLE_VERBOSE_DEBUG
+#ifdef SSFN_ENABLE_VERBOSE_DEBUG
     cuex_subst_dump(subst, stdout);
 #endif
     return cuex_ssfn_ctrl_commit;
@@ -1525,7 +1521,7 @@ cuex_ssfn_have_wpmgu(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key,
 	== cuex_ssfn_ctrl_commit;
 }
 
-cu_clop_def(ssfn_erase_cb, cuex_ssfn_ctrl_t,
+cu_clop_def(_ssfn_erase_cb, cuex_ssfn_ctrl_t,
 	    void *slot, cuex_subst_t subst,
 	    cu_rank_t pat_arg_cnt, cuex_t *pat_arg_arr,
 	    cuex_ssfn_matchinfo_t minfo)
@@ -1535,31 +1531,31 @@ cu_clop_def(ssfn_erase_cb, cuex_ssfn_ctrl_t,
 void
 cuex_ssfn_erase_lgr(cuex_ssfn_t ssfn, cuex_t key)
 {
-    cuex_ssfn_find_lgr(ssfn, NULL, key, 0, ssfn_erase_cb);
+    cuex_ssfn_find_lgr(ssfn, NULL, key, 0, _ssfn_erase_cb);
 }
 void
 cuex_ssfn_erase_mgr(cuex_ssfn_t ssfn, cuex_t key)
 {
-    cuex_ssfn_find_mgr(ssfn, NULL, key, 0, ssfn_erase_cb);
+    cuex_ssfn_find_mgr(ssfn, NULL, key, 0, _ssfn_erase_cb);
 }
 void
 cuex_ssfn_erase_wpmgr(cuex_ssfn_t ssfn, cuex_t key)
 {
-    cuex_ssfn_find_wpmgr(ssfn, NULL, key, 0, ssfn_erase_cb);
+    cuex_ssfn_find_wpmgr(ssfn, NULL, key, 0, _ssfn_erase_cb);
 }
 
 
 /* Find Single
  * ----------- */
 
-cu_clos_def(ssfn_find_single_cb,
-	    cuex_ssfn_ctrl_t
-	    cu_proto(void *slot, cuex_subst_t subst,
-		      cu_rank_t pat_arg_cnt, cuex_t *pat_arg_arr,
-		      cuex_ssfn_matchinfo_t minfo),
+cu_clos_def(_ssfn_find_single_cb,
+	    cu_prot(cuex_ssfn_ctrl_t,
+		    void *slot, cuex_subst_t subst,
+		    cu_rank_t pat_arg_cnt, cuex_t *pat_arg_arr,
+		    cuex_ssfn_matchinfo_t minfo),
 	    (void *slot;))
 {
-    cu_clos_self(ssfn_find_single_cb);
+    cu_clos_self(_ssfn_find_single_cb);
     self->slot = slot;
     return cuex_ssfn_ctrl_commit;
 }
@@ -1567,9 +1563,8 @@ cu_clos_def(ssfn_find_single_cb,
 void *
 cuex_ssfn_find_single_wpmgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key)
 {
-    ssfn_find_single_cb_t cb;
-    cu_clos_cct(&cb, ssfn_find_single_cb);
-    if (cuex_ssfn_find_wpmgr(ssfn, subst, key, 0, cu_clos_ref(&cb))
+    _ssfn_find_single_cb_t cb;
+    if (cuex_ssfn_find_wpmgr(ssfn, subst, key, 0, _ssfn_find_single_cb_prep(&cb))
 	== cuex_ssfn_ctrl_commit)
 	return cb.slot;
     else
@@ -1582,9 +1577,9 @@ cuex_ssfn_find_single_wpmgr(cuex_ssfn_t ssfn, cuex_subst_t subst, cuex_t key)
 
 typedef struct cuexP_ssfn_find_eqr_jargs_s
 {
-    struct cucon_stack_s input;
-    struct cucon_stack_s output;
-    struct cucon_pmap_s blocked_key_vars;
+    struct cucon_stack input;
+    struct cucon_stack output;
+    struct cucon_pmap blocked_key_vars;
     cuex_ssfn_ctrl_t ctrl;
     cu_clop(cb, cuex_ssfn_ctrl_t, void *);
 } *cuexP_ssfn_find_eqr_jargs_t;
@@ -1723,13 +1718,13 @@ cuex_ssfn_find_eqr(cuex_ssfn_t ssfn, cuex_t key,
 		  cu_clop(cb, cuex_ssfn_ctrl_t, void *))
 {
     struct cuexP_ssfn_find_eqr_jargs_s jargs;
-    cucon_stack_cct(&jargs.input);
-    cucon_stack_cct(&jargs.output);
+    cucon_stack_init(&jargs.input);
+    cucon_stack_init(&jargs.output);
     CUCON_STACK_PUSH(&jargs.input, cuex_t, key);
     jargs.ctrl = cuex_ssfn_ctrl_unmatched;
     jargs.cb = cb;
-    cucon_pmap_cct(&jargs.blocked_key_vars);
-    cuexP_ssfn_find_eqr(cu_upcast(cuex_ssfn_node_s, ssfn), &jargs);
+    cucon_pmap_init(&jargs.blocked_key_vars);
+    cuexP_ssfn_find_eqr(cu_to(cuex_ssfn_node, ssfn), &jargs);
     return jargs.ctrl;
 }
 
@@ -1737,14 +1732,14 @@ cuex_ssfn_find_eqr(cuex_ssfn_t ssfn, cuex_t key,
 /* Dump
  * ---- */
 
-cu_clos_def(ssfn_dump_keys_cb,
-	    cuex_ssfn_ctrl_t
-	    cu_proto(void *slot, cuex_subst_t subst,
-		      cu_rank_t pat_arg_cnt, cuex_t *pat_arg_arr,
-		      cuex_ssfn_matchinfo_t minfo),
+cu_clos_def(_ssfn_dump_keys_cb,
+	    cu_prot(cuex_ssfn_ctrl_t,
+		    void *slot, cuex_subst_t subst,
+		    cu_rank_t pat_arg_cnt, cuex_t *pat_arg_arr,
+		    cuex_ssfn_matchinfo_t minfo),
 	    (cuex_var_t var; FILE *out;))
 {
-    cu_clos_self(ssfn_dump_keys_cb);
+    cu_clos_self(_ssfn_dump_keys_cb);
     cu_fprintf(self->out, "    %!\n",
 	       cuex_subst_apply(subst, cuex_var_to_ex(self->var)));
     return cuex_ssfn_ctrl_continue;
@@ -1752,13 +1747,12 @@ cu_clos_def(ssfn_dump_keys_cb,
 void
 cuex_ssfn_dump_keys(cuex_ssfn_t ssfn, FILE *out)
 {
-    ssfn_dump_keys_cb_t cb;
-    cu_clos_cct(&cb, ssfn_dump_keys_cb);
+    _ssfn_dump_keys_cb_t cb;
     cb.var = cuex_var_new(cuex_qcode_active_s);
     cb.out = out;
-    fprintf(out, "DUMP OF ssfn @ %p:\n", ssfn);
+    fprintf(out, "DUMP OF ssfn @ %p:\n", (void *)ssfn);
     cuex_ssfn_find_lgr(ssfn, NULL, cuex_var_to_ex(cb.var), 0,
-		      cu_clos_ref(&cb));
+		       _ssfn_dump_keys_cb_prep(&cb));
     fputc('\n', out);
 }
 
