@@ -1,19 +1,18 @@
-/*-*-C-*-
- * Copyright 2004  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2004--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define CHI_SUBST_C
@@ -22,10 +21,11 @@
 #include <cucon/pmap.h>
 #include <cuex/ex.h>
 #include <cuex/opn.h>
-#include <cuex/opr.h>
-#include <cuex/aci.h>
+#include <cuex/oprdefs.h>
 #include <cuex/algo.h>
 #include <cuex/tvar.h>
+#include <cufo/stream.h>
+#include <cufo/tagdefs.h>
 
 
 /* Expensive debugging */
@@ -38,7 +38,7 @@
 cuex_veqv_t
 cuex_veqv_new(cuex_var_t var)
 {
-    cuex_veqv_t veqv = cu_gnew(struct cuex_veqv_s);
+    cuex_veqv_t veqv = cu_gnew(struct cuex_veqv);
     veqv->qcode = cuex_varmeta_qcode(cuex_meta(var));
     veqv->var_link = cucon_slink_prepend_ptr(cucon_slink_empty(), var);
     veqv->value = NULL;
@@ -48,7 +48,7 @@ cuex_veqv_new(cuex_var_t var)
 cuex_veqv_t
 cuex_veqv_new_copy(cuex_veqv_t src_veqv)
 {
-    cuex_veqv_t veqv = cu_gnew(struct cuex_veqv_s);
+    cuex_veqv_t veqv = cu_gnew(struct cuex_veqv);
     veqv->qcode = src_veqv->qcode;
     veqv->var_link = src_veqv->var_link;
     veqv->value = src_veqv->value;
@@ -65,13 +65,13 @@ cuex_veqv_unify(cuex_veqv_t veqv0, cuex_veqv_t veqv1, cuex_t new_value)
     cuex_var_t head_var;
     cucon_slink_t var_link;
     cucon_slink_t dst_var_link;
-    struct cucon_pmap_s varset;
+    struct cucon_pmap varset;
 /*     cu_debug_assert(veqv0->qcode == veqv1->qcode); /\* XXX *\/ */
     if (veqv1->qcode == cuex_qcode_active_w)
 	veqv1->qcode = veqv0->qcode;
 
     /* varset := vars(veqv1) */
-    cucon_pmap_cct(&varset);
+    cucon_pmap_init(&varset);
     var_link = veqv1->var_link;
     do {
 	cucon_pmap_insert_void(&varset, cucon_slink_get_ptr(var_link));
@@ -108,58 +108,58 @@ static void cuexP_subst_check(cuex_subst_t subst);
  * ------------ */
 
 CU_SINLINE void
-cuexP_subst_cct(cuex_subst_t subst, cuex_subst_t src_subst,
-		cuex_qcset_t qcset, cu_bool_t is_idem)
+cuexP_subst_init(cuex_subst_t subst, cuex_subst_t src_subst,
+		 cuex_qcset_t qcset, cu_bool_t is_idem)
 {
     subst->clone_cnt = 1;
     subst->shadow_access_cnt = 0;
     subst->shadowed = src_subst;
     subst->qcset = qcset;
     subst->is_idem = is_idem;
-    cucon_pmap_cct(&subst->var_to_veqv);
+    cucon_pmap_init(&subst->var_to_veqv);
 }
 
 void
-cuex_subst_cct(cuex_subst_t subst, cuex_qcset_t qcset)
+cuex_subst_init(cuex_subst_t subst, cuex_qcset_t qcset)
 {
-    cuexP_subst_cct(subst, NULL, qcset, cu_true);
+    cuexP_subst_init(subst, NULL, qcset, cu_true);
 }
 
 cuex_subst_t
 cuex_subst_new(cuex_qcset_t qcset)
 {
-    cuex_subst_t subst = cu_gnew(struct cuex_subst_s);
-    cuexP_subst_cct(subst, NULL, qcset, cu_true);
+    cuex_subst_t subst = cu_gnew(struct cuex_subst);
+    cuexP_subst_init(subst, NULL, qcset, cu_true);
     return subst;
 }
 
 void
-cuex_subst_cct_nonidem(cuex_subst_t subst, cuex_qcset_t qcset)
+cuex_subst_init_nonidem(cuex_subst_t subst, cuex_qcset_t qcset)
 {
-    cuexP_subst_cct(subst, NULL, qcset, cu_false);
+    cuexP_subst_init(subst, NULL, qcset, cu_false);
 }
 
 cuex_subst_t
 cuex_subst_new_nonidem(cuex_qcset_t qcset)
 {
-    cuex_subst_t subst = cu_gnew(struct cuex_subst_s);
-    cuexP_subst_cct(subst, NULL, qcset, cu_false);
+    cuex_subst_t subst = cu_gnew(struct cuex_subst);
+    cuexP_subst_init(subst, NULL, qcset, cu_false);
     return subst;
 }
 
 cuex_subst_t
 cuex_subst_new_clone(cuex_subst_t src_subst, cuex_qcset_t qcset)
 {
-    cuex_subst_t subst = cu_gnew(struct cuex_subst_s);
+    cuex_subst_t subst = cu_gnew(struct cuex_subst);
     while (src_subst && cucon_pmap_size(&src_subst->var_to_veqv) == 0)
 	src_subst = src_subst->shadowed;
     if (src_subst) {
-	cuexP_subst_cct(subst, src_subst, qcset, src_subst->is_idem);
+	cuexP_subst_init(subst, src_subst, qcset, src_subst->is_idem);
 	++src_subst->clone_cnt;
 	cu_debug_assert(qcset == src_subst->qcset);
     }
     else
-	cuexP_subst_cct(subst, src_subst, qcset, cu_true);
+	cuexP_subst_init(subst, src_subst, qcset, cu_true);
     return subst;
 }
 
@@ -196,25 +196,25 @@ cuex_subst_delete(cuex_subst_t subst)
 /* Lookup
  * ------ */
 
-cu_clos_def(cuexP_subst_merge_cb_move,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_subst_merge_cb_move,
+	    cu_prot(void, void const *var, void *veqv),
 	    (cucon_pmap_t dst;))
 {
-    cu_clos_self(cuexP_subst_merge_cb_move);
+    cu_clos_self(_subst_merge_cb_move);
     cucon_pmap_replace_ptr(self->dst, var, veqv);
 }
-cu_clos_def(cuexP_subst_merge_cb_copy,
-	    void cu_proto(void const *var0, void *veqv0),
+cu_clos_def(_subst_merge_cb_copy,
+	    cu_prot(void, void const *var0, void *veqv0),
 	    (cucon_pmap_t dst;))
 {
 #define var ((cuex_var_t)var0)
 #define veqv ((cuex_veqv_t)veqv0)
-    cu_clos_self(cuexP_subst_merge_cb_copy);
+    cu_clos_self(_subst_merge_cb_copy);
     cucon_slink_t var_cur = veqv->var_link;
     if (var == cucon_slink_get_ptr(var_cur)) {
 	cuex_veqv_t *slot;
 	if (cucon_pmap_insert_mem(self->dst, var,
-				sizeof(struct cuex_veqv_s), &slot)) {
+				sizeof(struct cuex_veqv), &slot)) {
 	    veqv0 = cuex_veqv_new_copy(veqv);
 	    *slot = veqv;
 	    var_cur = cucon_slink_next(var_cur);
@@ -227,7 +227,7 @@ cu_clos_def(cuexP_subst_merge_cb_copy,
     }
 #undef var
 #undef veqv
-};  
+}
 
 static void
 cuexP_subst_merge(cuex_subst_t dst)
@@ -237,16 +237,14 @@ cuexP_subst_merge(cuex_subst_t dst)
 	if (cucon_pmap_size(&src->var_to_veqv)
 	    > cucon_pmap_size(&dst->var_to_veqv))
 	    cucon_pmap_swap(&src->var_to_veqv, &dst->var_to_veqv);
-	cuexP_subst_merge_cb_move_t cb;
-	cu_clos_cct(&cb, cuexP_subst_merge_cb_move);
+	_subst_merge_cb_move_t cb;
 	cb.dst = &dst->var_to_veqv;
-	cucon_pmap_iter_ptr(&src->var_to_veqv, cu_clos_ref(&cb));
+	cucon_pmap_iter_ptr(&src->var_to_veqv, _subst_merge_cb_move_prep(&cb));
     }
     else {
-	cuexP_subst_merge_cb_copy_t cb;
-	cu_clos_cct(&cb, cuexP_subst_merge_cb_copy);
+	_subst_merge_cb_copy_t cb;
 	cb.dst = &dst->var_to_veqv;
-	cucon_pmap_iter_ptr(&src->var_to_veqv, cu_clos_ref(&cb));
+	cucon_pmap_iter_ptr(&src->var_to_veqv, _subst_merge_cb_copy_prep(&cb));
     }
     dst->shadowed = src->shadowed;
 #ifdef CHECK_SUBST
@@ -318,7 +316,7 @@ cuex_subst_mref(cuex_subst_t subst, cuex_var_t var)
     return *slot;
 }
 
-cu_clop_def(subst_is_identity_cb, cu_bool_t, void const *var, void *veqv)
+cu_clop_def(_subst_is_identity_cb, cu_bool_t, void const *var, void *veqv)
 {
 #define veqv ((cuex_veqv_t)veqv)
     return veqv->value == NULL && cucon_slink_next(veqv->var_link) == NULL;
@@ -329,7 +327,7 @@ cu_bool_t
 cuex_subst_is_identity(cuex_subst_t subst)
 {
     while (subst)
-	if (!cucon_pmap_conj_ptr(&subst->var_to_veqv, subst_is_identity_cb))
+	if (!cucon_pmap_conj_ptr(&subst->var_to_veqv, _subst_is_identity_cb))
 	    return cu_false;
     return cu_true;
 }
@@ -513,11 +511,11 @@ cuex_subst_block(cuex_subst_t subst, cuex_var_t v)
 {
     cuex_veqv_t vq = cuex_subst_mref(subst, v);
     if (vq->value) {
-	cu_debug_assert(cuex_meta(vq->value) == cuex_opr_subst_block_1);
+	cu_debug_assert(cuex_meta(vq->value) == CUEX_O1_SUBST_BLOCK);
 	return;
     }
     vq->value = cuex_opn_to_ex(
-	cuex_opn(cuex_opr_subst_block_1, cuex_var_to_ex(cuex_var_new_n())));
+	cuex_opn(CUEX_O1_SUBST_BLOCK, cuex_var_to_ex(cuex_var_new_n())));
 }
 
 void
@@ -525,35 +523,35 @@ cuex_subst_unblock(cuex_subst_t subst, cuex_var_t v)
 {
     cuex_veqv_t vq = cuex_subst_mref(subst, v);
     cu_debug_assert(vq->value &&
-		     cuex_meta(vq->value) == cuex_opr_subst_block_1);
+		     cuex_meta(vq->value) == CUEX_O1_SUBST_BLOCK);
     vq->value = NULL;
 }
 
 
 /* -- cuex_subst_unblock_all */
 
-cu_clos_def(unblock_all_cb0,
-	    void cu_proto(void const *var, void *veqv),
-	    ( struct cucon_pmap_s visited; ) )
+cu_clos_def(_unblock_all_cb0,
+	    cu_prot(void, void const *var, void *veqv),
+	    ( struct cucon_pmap visited; ) )
 {
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(unblock_all_cb0);
+    cu_clos_self(_unblock_all_cb0);
     if (!cucon_pmap_insert_void(&self->visited, (cuex_var_t)var))
 	return;
-    if (veqv->value && cuex_meta(veqv->value) == cuex_opr_subst_block_1)
+    if (veqv->value && cuex_meta(veqv->value) == CUEX_O1_SUBST_BLOCK)
 	veqv->value = NULL;
 #undef veqv
 }
-cu_clos_def(unblock_all_cb1,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_unblock_all_cb1,
+	    cu_prot(void, void const *var, void *veqv),
 	    ( cucon_pmap_t visited;
 	      cuex_subst_t subst; ) )
 {
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(unblock_all_cb1);
+    cu_clos_self(_unblock_all_cb1);
     if (!cucon_pmap_insert_void(self->visited, var))
 	return;
-    if (veqv->value && cuex_meta(veqv->value) == cuex_opr_subst_block_1)
+    if (veqv->value && cuex_meta(veqv->value) == CUEX_O1_SUBST_BLOCK)
 	cuex_subst_mref(self->subst, (cuex_var_t)var)->value = NULL;
 #undef veqv
 }
@@ -561,17 +559,16 @@ void
 cuex_subst_unblock_all(cuex_subst_t subst)
 {
     if (subst) {
-	unblock_all_cb0_t cb0;
-	cu_clos_cct(&cb0, unblock_all_cb0);
-	cucon_pmap_cct(&cb0.visited);
-	cucon_pmap_iter_ptr(&subst->var_to_veqv, cu_clos_ref(&cb0));
+	_unblock_all_cb0_t cb0;
+	cucon_pmap_init(&cb0.visited);
+	cucon_pmap_iter_ptr(&subst->var_to_veqv, _unblock_all_cb0_prep(&cb0));
 	subst = subst->shadowed;
 	while (subst) {
-	    unblock_all_cb1_t cb1;
-	    cu_clos_cct(&cb1, unblock_all_cb1);
+	    _unblock_all_cb1_t cb1;
 	    cb1.subst = subst;
 	    cb1.visited = &cb0.visited;
-	    cucon_pmap_iter_ptr(&subst->var_to_veqv, cu_clos_ref(&cb1));
+	    cucon_pmap_iter_ptr(&subst->var_to_veqv,
+				_unblock_all_cb1_prep(&cb1));
 	    subst = subst->shadowed;
 	}
     }
@@ -580,11 +577,11 @@ cuex_subst_unblock_all(cuex_subst_t subst)
 
 /* -- cuex_subst_freshen_and_block_vars_in */
 
-cu_clos_def(subst_freshen_var_cb,
-	    cu_bool_t cu_proto(cuex_var_t var),
+cu_clos_def(_subst_freshen_var_cb,
+	    cu_prot(cu_bool_t, cuex_var_t var),
 	    (cuex_subst_t subst;))
 {
-    cu_clos_self(subst_freshen_var_cb);
+    cu_clos_self(_subst_freshen_var_cb);
     cuex_subst_t subst = self->subst;
     cuex_meta_t var_meta = cuex_meta(var);
     if (cuex_is_varmeta(var_meta)
@@ -603,10 +600,9 @@ cu_clos_def(subst_freshen_var_cb,
 void
 cuex_subst_freshen_and_block_vars_in(cuex_subst_t subst, cuex_t ex)
 {
-    subst_freshen_var_cb_t cb;
-    cu_clos_cct(&cb, subst_freshen_var_cb);
+    _subst_freshen_var_cb_t cb;
     cb.subst = subst;
-    cuex_depth_conj_vars(ex, cu_clos_ref(&cb));
+    cuex_depth_conj_vars(ex, _subst_freshen_var_cb_prep(&cb));
 }
 
 
@@ -618,7 +614,7 @@ cuex_subst_lookup(cuex_subst_t subst, cuex_var_t var)
     cuex_veqv_t vq = cuex_subst_cref(subst, var);
     if (!vq)
 	return NULL;
-    else if (vq->value && cuex_meta(vq->value) != cuex_opr_subst_block_1)
+    else if (vq->value && cuex_meta(vq->value) != CUEX_O1_SUBST_BLOCK)
 	return vq->value;
     else if (cucon_slink_next(vq->var_link))
 	return cucon_slink_get_ptr(vq->var_link);
@@ -630,11 +626,11 @@ cuex_subst_lookup(cuex_subst_t subst, cuex_var_t var)
 /* -- cuex_subst_apply */
 
 static cuex_t cuexP_subst_apply(cuex_subst_t, cuex_t);
-cu_clos_def(subst_apply_cb,
-	    cuex_opn_t cu_proto(cuex_opn_t node),
+cu_clos_def(_subst_apply_cb,
+	    cu_prot(cuex_opn_t, cuex_opn_t node),
 	( cuex_subst_t subst; ))
 {
-    cu_clos_self(subst_apply_cb);
+    cu_clos_self(_subst_apply_cb);
     cuex_meta_t meta = cuex_meta(node);
     CUEX_OPN_TRAN(meta, node, subex, cuexP_subst_apply(self->subst, subex));
     return node;
@@ -650,7 +646,7 @@ tailcall:
 	vq = cuex_subst_cref(subst, cuex_var_from_ex(ex));
 	if (vq) {
 	    if (vq->value) {
-		if (cuex_meta(vq->value) == cuex_opr_subst_block_1)
+		if (cuex_meta(vq->value) == CUEX_O1_SUBST_BLOCK)
 		    return ex;
 		ex = vq->value;
 		goto tailcall;
@@ -661,13 +657,14 @@ tailcall:
 	return ex;
     }
     if (cuex_meta_is_opr(meta)) {
+#if 0
 	if (cuex_opr_is_aci(meta)) {
-	    subst_apply_cb_t cb;
-	    cu_clos_cct(&cb, subst_apply_cb);
+	    _subst_apply_cb_t cb;
 	    cb.subst = subst;
-	    return cuex_aci_tran(meta, ex, cu_clos_ref(&cb));
+	    return cuex_aci_tran(meta, ex, _subst_apply_cb_prep(&cb));
 	}
 	else
+#endif
 	    CUEX_OPN_TRAN(meta, ex, subex, cuexP_subst_apply(subst, subex));
     }
     return ex;
@@ -689,11 +686,11 @@ cuex_subst_apply(cuex_subst_t subst, cuex_t ex)
 
 /* -- cuex_subst_update_tvar_types */
 
-cu_clos_def(subst_update_tvar_types_cb,
-	    cu_bool_t cu_proto(cuex_var_t var),
+cu_clos_def(_subst_update_tvar_types_cb,
+	    cu_prot(cu_bool_t, cuex_var_t var),
 	( cuex_subst_t subst; ))
 {
-    cu_clos_self(subst_update_tvar_types_cb);
+    cu_clos_self(_subst_update_tvar_types_cb);
     if (cuex_is_tvar(var)) {
 	cuex_t t = cuex_tvar_type(cuex_tvar_from_var(var));
 	if (t)
@@ -706,10 +703,9 @@ cu_clos_def(subst_update_tvar_types_cb,
 void
 cuex_subst_update_tvar_types(cuex_subst_t subst, cuex_t ex)
 {
-    subst_update_tvar_types_cb_t cb;
-    cu_clos_cct(&cb, subst_update_tvar_types_cb);
+    _subst_update_tvar_types_cb_t cb;
     cb.subst = subst;
-    cuex_depth_conj_vars(ex, cu_clos_ref(&cb));
+    cuex_depth_conj_vars(ex, _subst_update_tvar_types_cb_prep(&cb));
 }
 
 
@@ -723,7 +719,7 @@ cuexP_subst_expand(cuex_subst_t subst, cuex_t ex)
 	cuex_veqv_t vq = cuex_subst_cref(subst, cuex_var_from_ex(ex));
 	if (vq) {
 	    if (vq->value) {
-		if (cuex_meta(vq->value) == cuex_opr_subst_block_1)
+		if (cuex_meta(vq->value) == CUEX_O1_SUBST_BLOCK)
 		    return ex;
 		else
 		    return vq->value;
@@ -810,7 +806,7 @@ recurse:
 		veqv = cuex_subst_cref(subst, cuex_var_from_ex(ex));
 		if (veqv) {
 		    if (veqv->value &&
-			cuex_meta(veqv->value) != cuex_opr_subst_block_1) {
+			cuex_meta(veqv->value) != CUEX_O1_SUBST_BLOCK) {
 			ex = veqv->value;
 			goto recurse;
 		    }
@@ -831,32 +827,32 @@ void
 cuex_subst_free_vars_insert(cuex_subst_t subst, cuex_t ex, cucon_pset_t accu)
 {
     cuex_pset_curried_insert_ex_t cb;
-    cu_clos_cct(&cb, cuex_pset_curried_insert_ex);
     cb.accu = accu;
     cuex_subst_free_vars_conj(subst, ex,
-	    cu_clos_ref_cast(cu_bool_t cu_proto(cuex_var_t), &cb));
+			      (cu_clop(, cu_bool_t, cuex_var_t))
+				    cuex_pset_curried_insert_ex_prep(&cb));
 }
 
 void
 cuex_subst_free_vars_erase(cuex_subst_t subst, cuex_t ex, cucon_pset_t accu)
 {
     cuex_pset_curried_erase_ex_t cb;
-    cu_clos_cct(&cb, cuex_pset_curried_erase_ex);
     cb.accu = accu;
     cuex_subst_free_vars_conj(subst, ex,
-	    cu_clos_ref_cast(cu_bool_t cu_proto(cuex_var_t), &cb));
+			      (cu_clop(, cu_bool_t, cuex_var_t))
+				    cuex_pset_curried_erase_ex_prep(&cb));
 }
 
 
 /* -- cuex_subst_dump */
 
-cu_clos_def(cuex_subst_dump_cb,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_subst_dump_cb,
+	    cu_prot(void, void const *var, void *veqv),
 	    (FILE *file;))
 {
 #define var ((cuex_var_t)var)
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(cuex_subst_dump_cb);
+    cu_clos_self(_subst_dump_cb);
     cucon_slink_t var_cur;
     FILE *file = self->file;
     if (cucon_slink_get_ptr(veqv->var_link) != var)
@@ -867,7 +863,7 @@ cu_clos_def(cuex_subst_dump_cb,
 	cu_fprintf(file, ", %!", cuex_var_to_ex(cucon_slink_get_ptr(var_cur)));
     fprintf(file, "]");
     if (veqv->value) {
-	if (cuex_meta(veqv->value) == cuex_opr_subst_block_1)
+	if (cuex_meta(veqv->value) == CUEX_O1_SUBST_BLOCK)
 	    fprintf(file, " = __blocked_%p", veqv->value);
 	else
 	    cu_fprintf(file, " = %!", veqv->value);
@@ -879,14 +875,15 @@ cu_clos_def(cuex_subst_dump_cb,
 void
 cuex_subst_dump(cuex_subst_t subst, FILE *file)
 {
-    cuex_subst_dump_cb_t cb;
-    cu_clos_cct(&cb, cuex_subst_dump_cb);
+    _subst_dump_cb_t cb;
+    cu_clop(cb_clop, void, void const *, void *)
+	= _subst_dump_cb_prep(&cb);
     cb.file = file;
-    fprintf(file, "subst @ %p (qcset = %x, %s)\n", subst,
+    fprintf(file, "subst @ %p (qcset = %x, %s)\n", (void *)subst,
 	    subst->qcset, subst->is_idem? "idem" : "nonidem");
     while (subst) {
-	fprintf(file, "    frame @ %p\n", subst);
-	cucon_pmap_iter_ptr(&subst->var_to_veqv, cu_clos_ref(&cb));
+	fprintf(file, "    frame @ %p\n", (void *)subst);
+	cucon_pmap_iter_ptr(&subst->var_to_veqv, cb_clop);
 	subst = subst->shadowed;
     }
 }
@@ -894,12 +891,12 @@ cuex_subst_dump(cuex_subst_t subst, FILE *file)
 
 /* -- cuex_subst_iter_veqv */
 
-cu_clos_def(cuex_subst_iter_cb,
-	    void cu_proto(void const *var, void *veqv),
-	    (cu_clop(cb, void, cuex_veqv_t); struct cucon_pmap_s visited;))
+cu_clos_def(_subst_iter_cb,
+	    cu_prot(void, void const *var, void *veqv),
+	    (cu_clop(cb, void, cuex_veqv_t); struct cucon_pmap visited;))
 {
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(cuex_subst_iter_cb);
+    cu_clos_self(_subst_iter_cb);
     if (cucon_pmap_insert_void(&self->visited, var)
 	&& var == cucon_slink_get_ptr(veqv->var_link)
 	&& (veqv->value || cucon_slink_next(veqv->var_link)))
@@ -909,12 +906,13 @@ cu_clos_def(cuex_subst_iter_cb,
 void
 cuex_subst_iter_veqv(cuex_subst_t subst, cu_clop(cb, void, cuex_veqv_t))
 {
-    cuex_subst_iter_cb_t cb0;
-    cu_clos_cct(&cb0, cuex_subst_iter_cb);
+    _subst_iter_cb_t cb0;
+    cu_clop(cb0_clop, void, void const *, void *)
+	= _subst_iter_cb_prep(&cb0);
     cb0.cb = cb;
-    cucon_pmap_cct(&cb0.visited);
+    cucon_pmap_init(&cb0.visited);
     while (subst) {
-	cucon_pmap_iter_ptr(&subst->var_to_veqv, cu_clos_ref(&cb0));
+	cucon_pmap_iter_ptr(&subst->var_to_veqv, cb0_clop);
 	subst = subst->shadowed;
     }
 }
@@ -923,16 +921,16 @@ cuex_subst_iter_veqv(cuex_subst_t subst, cu_clop(cb, void, cuex_veqv_t))
 /* -- cuex_subst_iter_veqv_cache
  * -- cuex_subst_iter_veqv_cache_sub */
 
-cu_clos_def(subst_iter_veqv_cache_cb,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_subst_iter_veqv_cache_cb,
+	    cu_prot(void, void const *var, void *veqv),
     (	cu_clop(f, void, cuex_veqv_t veqv, void *cache, void *sub_data);
-	struct cucon_pmap_s cache;
+	struct cucon_pmap cache;
 	size_t cache_size;
 	cuex_subst_t sig;
     ))
 {
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(subst_iter_veqv_cache_cb);
+    cu_clos_self(_subst_iter_veqv_cache_cb);
     if (veqv->value || cucon_slink_next(veqv->var_link)) {
 	void *cache;
 	if (cucon_pmap_insert_mem(&self->cache, veqv,
@@ -947,15 +945,16 @@ cuex_subst_iter_veqv_cache(cuex_subst_t sig, size_t cache_size,
 			   cu_clop(f, void, cuex_veqv_t veqv, void *cache,
 					    void *sub_data))
 {
-    subst_iter_veqv_cache_cb_t cb0;
+    _subst_iter_veqv_cache_cb_t cb0;
+    cu_clop(cb0_clop, void, void const *, void *)
+	= _subst_iter_veqv_cache_cb_prep(&cb0);
     if (!sig) return;
-    cu_clos_cct(&cb0, subst_iter_veqv_cache_cb);
     cb0.f = f;
-    cucon_pmap_cct(&cb0.cache);
+    cucon_pmap_init(&cb0.cache);
     cb0.cache_size = cache_size;
     cb0.sig = sig;
     do {
-	cucon_pmap_iter_ptr(&sig->var_to_veqv, cu_clos_ref(&cb0));
+	cucon_pmap_iter_ptr(&sig->var_to_veqv, cb0_clop);
 	sig = sig->shadowed;
     } while (sig);
 }
@@ -963,7 +962,7 @@ cuex_subst_iter_veqv_cache(cuex_subst_t sig, size_t cache_size,
 void *
 cuex_subst_iter_veqv_cache_sub(void *sub_data, cuex_t var)
 {
-#define self ((subst_iter_veqv_cache_cb_t *)sub_data)
+#define self ((_subst_iter_veqv_cache_cb_t *)sub_data)
     cuex_veqv_t veqv = cuex_subst_cref(self->sig, var);
     if (veqv && (veqv->value || cucon_slink_next(veqv->var_link))) {
 	void *cache;
@@ -985,10 +984,10 @@ cuex_subst_iter_veqv_cache_sub(void *sub_data, cuex_t var)
  * substitutions, else use a fresh substitution. */
 #define CUEXOPT_SUBST_TRAN_CLONE 1
 
-cu_clos_def(subst_tran_cache_cb,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_subst_tran_cache_cb,
+	    cu_prot(void, void const *var, void *veqv),
     (	cu_clop(f, cuex_t, cuex_t value, void *cache, void *sub_data);
-	struct cucon_pmap_s cache;
+	struct cucon_pmap cache;
 	size_t cache_size;
 	cuex_subst_t sig;
 	cuex_subst_t tau;
@@ -996,7 +995,7 @@ cu_clos_def(subst_tran_cache_cb,
 {
 #define veqv ((cuex_veqv_t)veqv)
 #define var ((cuex_t)var)
-    cu_clos_self(subst_tran_cache_cb);
+    cu_clos_self(_subst_tran_cache_cb);
     if (veqv->value) {
 	void *cache;
 	if (cucon_pmap_insert_mem(&self->cache, veqv,
@@ -1024,11 +1023,12 @@ cuex_subst_tran_cache(cuex_subst_t sig, size_t cache_size,
 		      cu_clop(f, cuex_t, cuex_t value, void *cache,
 					 void *sub_data))
 {
-    subst_tran_cache_cb_t cb;
+    _subst_tran_cache_cb_t cb;
+    cu_clop(cb_clop, void, void const *, void *)
+	= _subst_tran_cache_cb_prep(&cb);
     if (!sig) return NULL;
-    cu_clos_cct(&cb, subst_tran_cache_cb);
     cb.f = f;
-    cucon_pmap_cct(&cb.cache);
+    cucon_pmap_init(&cb.cache);
     cb.cache_size = cache_size;
     cb.sig = sig;
 #if CUEXOPT_SUBST_TRAN_CLONE
@@ -1037,7 +1037,7 @@ cuex_subst_tran_cache(cuex_subst_t sig, size_t cache_size,
     cb.tau = cuex_subst_new(cuex_subst_active_qcset(sig));
 #endif
     do {
-	cucon_pmap_iter_ptr(&sig->var_to_veqv, cu_clos_ref(&cb));
+	cucon_pmap_iter_ptr(&sig->var_to_veqv, cb_clop);
 	sig = sig->shadowed;
     } while (sig);
     return cb.tau;
@@ -1046,7 +1046,7 @@ cuex_subst_tran_cache(cuex_subst_t sig, size_t cache_size,
 void *
 cuex_subst_tran_cache_sub(void *sub_data, cuex_t var)
 {
-#define self ((subst_tran_cache_cb_t *)sub_data)
+#define self ((_subst_tran_cache_cb_t *)sub_data)
     cuex_veqv_t veqv = cuex_subst_cref(self->sig, var);
     if (veqv && veqv->value) {
 	void *cache;
@@ -1076,43 +1076,42 @@ cuex_subst_tran_cache_sub(void *sub_data, cuex_t var)
 
 /* -- cuex_subst_print */
 
-cu_clos_def(cuex_subst_print_cb,
-	    void cu_proto(cuex_veqv_t veqv),
-	    ( FILE *file;
+cu_clos_def(_subst_print_cb,
+	    cu_prot(void, cuex_veqv_t veqv),
+	    ( cufo_stream_t fos;
 	      char const *sep;
 	      cu_bool_t is_first; ))
 {
-    cu_clos_self(cuex_subst_print_cb);
+    cu_clos_self(_subst_print_cb);
     cucon_slink_t var_cur;
     char *sep;
     if (self->is_first)
 	self->is_first = cu_false;
     else
-	fputs(self->sep, self->file);
+	cufo_puts(self->fos, self->sep);
     sep = "";
     for (var_cur = veqv->var_link; var_cur;
 	 var_cur = cucon_slink_next(var_cur)) {
-	cu_fprintf(self->file, "%s%!",
-		   sep, cuex_var_to_ex(cucon_slink_get_ptr(var_cur)));
+	cufo_printf(self->fos, "%s%!",
+		    sep, cuex_var_to_ex(cucon_slink_get_ptr(var_cur)));
 	sep = " = ";
     }
     if (veqv->value) {
-	if (cuex_meta(veqv->value) == cuex_opr_subst_block_1)
-	    fprintf(self->file, " __blocked_%p", veqv->value);
+	if (cuex_meta(veqv->value) == CUEX_O1_SUBST_BLOCK)
+	    cufo_printf(self->fos, " __blocked_%p", veqv->value);
 	else
-	    cu_fprintf(self->file, " ↦ %!", veqv->value);
+	    cufo_printf(self->fos, " %<↦%> %!", cufoT_operator, veqv->value);
     }
 }
 
 void
-cuex_subst_print(cuex_subst_t subst, FILE *file, char const *sep)
+cuex_subst_print(cuex_subst_t subst, cufo_stream_t fos, char const *sep)
 {
-    cuex_subst_print_cb_t cb;
-    cu_clos_cct(&cb, cuex_subst_print_cb);
-    cb.file = file;
+    _subst_print_cb_t cb;
+    cb.fos = fos;
     cb.sep = sep;
     cb.is_first = cu_true;
-    cuex_subst_iter_veqv(subst, cu_clos_ref(&cb));
+    cuex_subst_iter_veqv(subst, _subst_print_cb_prep(&cb));
 }
 
 
@@ -1142,13 +1141,13 @@ cuexP_subst_check_ex(cuex_subst_t subst, cuex_t ex, cucon_pmap_t seen)
     }
 }
 
-cu_clos_def(subst_check_cb,
-	    void cu_proto(void const *var, void *veqv),
+cu_clos_def(_subst_check_cb,
+	    cu_prot(void, void const *var, void *veqv),
     ( cuex_subst_t subst; ) )
 {
 #define var ((cuex_var_t)var)
 #define veqv ((cuex_veqv_t)veqv)
-    cu_clos_self(subst_check_cb);
+    cu_clos_self(_subst_check_cb);
     cuex_subst_t subst = self->subst;
     cucon_slink_t var_cur;
 
@@ -1173,8 +1172,8 @@ cu_clos_def(subst_check_cb,
 
 	/* Occur check. */
 	if (veqv->value) {
-	    struct cucon_pmap_s seen;
-	    cucon_pmap_cct(&seen);
+	    struct cucon_pmap seen;
+	    cucon_pmap_init(&seen);
 	    cucon_pmap_insert_void(&seen, veqv);
 	    cuexP_subst_check_ex(subst, veqv->value, &seen);
 	}
@@ -1187,10 +1186,9 @@ static void
 cuexP_subst_check(cuex_subst_t subst)
 {
     if (subst) {
-	subst_check_cb_t cb;
-	cu_clos_cct(&cb, subst_check_cb);
+	_subst_check_cb_t cb;
 	cb.subst = subst;
-	cucon_pmap_iter_ptr(&subst->var_to_veqv, cu_clos_ref(&cb));
+	cucon_pmap_iter_ptr(&subst->var_to_veqv, _subst_check_cb_prep(&cb));
     }
 }
 #endif
