@@ -1,31 +1,33 @@
-/*-*-C-*-
- * Copyright 2006  Petter Urkedal
+/* Part of the culibs project, <http://www.eideticdew.org/culibs/>.
+ * Copyright (C) 2006--2007  Petter Urkedal <urkedal@nbi.dk>
  *
- * This file is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <cugra/fwd.h>
+#include <cu/diag.h>
+#ifdef CUCONF_HAVE_BUDDY
 #include <bdd.h>
+#endif
 
+#ifdef CUCONF_HAVE_BUDDY
 static void
-bdd_error(int err)
+_bdd_error(int err)
 {
     cu_bugf("BDD error %s", bdd_errstring(err));
 }
-
-void cugraP_graph_init(void);
+#endif
 
 void
 cugra_init(void)
@@ -36,9 +38,10 @@ cugra_init(void)
     done_init = 1;
 
     cu_init();
-    cugraP_graph_init();
 
+#ifdef CUCONF_HAVE_BUDDY
     bdd_init(10000, 1000);
-    bdd_error_hook(bdd_error);
+    bdd_error_hook(_bdd_error);
     bdd_setcacheratio(10);
+#endif
 }
